'''
Created on Nov 26, 2017

@author: fan

Most type of state grid generation:
    Given N Vectors,
'''
import logging

logger = logging.getLogger(__name__)

'''
Created on Mar 17, 2017

@author: fan
'''

import numpy as np


def two_mat_mesh(mat_one, mat_two, return_joint=False, return_single_col=False):
    """
    Parameters
    ----------
    return_single_col: boolean
        mat_one and mat_two are single vector, shape them into 2d array with
        1 column, rather than 1d. If not, could cause multiplication problems
        when we have both 1 column 2d array and single column 1d array in the
        same formula. But this can not always to set to True, hence default
        is actually false, because this function could take as input a matrix
        for mat_one, in that case, already 2d array.
    """
    logger.debug('return_joint:%s', return_joint)
    logger.debug('return_single_col:%s', return_single_col)

    mat_one_len = check_length(mat_one)
    mat_two_len = check_length(mat_two)
    logger.debug('mat_one_len:%s', mat_one_len)
    logger.debug('mat_one:\n%s', mat_one)
    logger.debug('mat_two_len:%s', mat_two_len)
    logger.debug('mat_two:\n%s', mat_two)

    mat_one_meshed = np.repeat(mat_one, mat_two_len, axis=0)
    mat_two_meshed = np.tile(mat_two, (mat_one_len, 1))

    if (return_joint == True):
        return np.concatenate((mat_one_meshed, mat_two_meshed), axis=1)
    else:
        if (return_single_col):
            mat_one_meshed = np.reshape(mat_one_meshed, (-1, 1))
            mat_two_meshed = np.reshape(mat_two_meshed, (-1, 1))

        return mat_one_meshed, mat_two_meshed


def three_mat_mesh(mat_one, mat_two, mat_three, return_joint=False, return_single_col=False):
    """
    Parameters
    ----------
    return_single_col: boolean
        mat_one and mat_two are single vector, shape them into 2d array with
        1 column, rather than 1d. If not, could cause multiplication problems
        when we have both 1 column 2d array and single column 1d array in the
        same formula. But this can not always to set to True, hence default
        is actually false, because this function could take as input a matrix
        for mat_one, in that case, already 2d array.
    """

    mat_one_len = check_length(mat_one)
    mat_two_len = check_length(mat_two)
    mat_three_len = check_length(mat_three)

    mat_one_meshed, mat_two_meshed = two_mat_mesh(mat_one, mat_two)

    mat_one_meshed_threed = np.repeat(mat_one_meshed, mat_three_len, axis=0)
    mat_two_meshed_threed = np.repeat(mat_two_meshed, mat_three_len, axis=0)

    # To increase sensitivity of choices and likelihood to changing parameters, turn this into a ratio
    mat_three_mesh_threed = np.tile(mat_three, ((mat_one_len * mat_two_len), 1))

    if (return_joint == True):
        return np.concatenate((mat_one_meshed_threed,
                               mat_two_meshed_threed,
                               mat_three_mesh_threed), axis=1)
    else:
        if (return_single_col):
            mat_one_meshed_threed = np.reshape(mat_one_meshed_threed, (-1, 1))
            mat_two_meshed_threed = np.reshape(mat_two_meshed_threed, (-1, 1))
            mat_three_mesh_threed = np.reshape(mat_three_mesh_threed, (-1, 1))

        return mat_one_meshed_threed, mat_two_meshed_threed, mat_three_mesh_threed


def multipl_mat_mesh(mat_one, mat_two, mat_three=None,
                     mat_four=None, mat_five=None, mat_six=None):
    mat_left = two_mat_mesh(mat_one, mat_two, return_joint=True)
    for mat_ctr in range(3, 6, 1):
        if (mat_ctr == 3):
            mat_left = two_mat_mesh(mat_left, mat_three, return_joint=True)
        if (mat_ctr == 4):
            mat_left = two_mat_mesh(mat_left, mat_four, return_joint=True)
        if (mat_ctr == 5):
            mat_left = two_mat_mesh(mat_left, mat_five, return_joint=True)
        if (mat_ctr == 6):
            mat_left = two_mat_mesh(mat_left, mat_six, return_joint=True)

    return mat_left


def check_length(mat):
    if (mat.ndim == 1):
        mat_len = len(mat)
        # mat = np.reshape(mat, (mat_len,1))
    else:
        mat_len = len(mat[:, 0])
    return mat_len  # mat


if __name__ == '__main__':
    print('Two Matrix Mesh')
    mat_one = np.array([[1, 2, 3], [3, 4, 5]])
    mat_two = np.array([[2.1, 3.2], [3.5, 4.5], [5.5, 9.5]])
    mat_one_two_joint = two_mat_mesh(mat_one, mat_two, return_joint=True)
    print(mat_one_two_joint)

    print('Three Matrix Mesh')
    mat_one = np.array([[1, 2, 3], [3, 4, 5]])
    mat_two = np.floor(np.random.rand(2, 1) * 100)
    mat_three = np.array([[2.1, 3.2], [3.5, 4.5], [5.5, 9.5]])
    mat_one_two_joint = three_mat_mesh(mat_one, mat_two, mat_three, return_joint=True)
    print(mat_one_two_joint)

    print('Two horizontal array Mesh')
    mat_one = np.array([[1, 2, 3]])
    mat_two = np.array([[2.1, 3.2]])
    mat_one_two_joint = two_mat_mesh(mat_one, mat_two, return_joint=True)
    print(mat_one_two_joint)

    print('Two vertical array Mesh')
    mat_one = np.array([[1], [2], [3]])
    mat_two = np.array([[2.1], [3.2]])
    mat_one_two_joint = two_mat_mesh(mat_one, mat_two, return_joint=True)
    print(mat_one_two_joint)

    print('Horizontal and Vertical Arrays Mesh')
    mat_one = np.array([[1], [2], [3]])
    mat_two = np.array([[2.1, 3.2]])
    mat_one_two_joint = two_mat_mesh(mat_one, mat_two, return_joint=True)
    print(mat_one_two_joint)

    print('linspace arrays mesh')
    mat_one = np.reshape(np.linspace(1, 2, 5), (-1, 1))
    mat_two = np.reshape(np.linspace(-2, -1, 2), (-1, 1))
    mat_one_two_joint = two_mat_mesh(mat_one, mat_two, return_joint=True)
    print(mat_one_two_joint)

    print('linspace arrays mesh')
    mat_one = np.reshape(np.linspace(1, 2, 5), (-1, 1))
    mat_two = np.reshape(np.linspace(-2, -1, 2), (-1, 1))
    mat_three = np.array([[1], [2], [3]])
    mat_four = np.array([[2.1], [3.2]])
    mat_five = np.array([[2.1, 3.2], [3.5, 4.5], [5.5, 9.5]])

    mat_joint = multipl_mat_mesh(mat_one, mat_two, mat_three, mat_four, mat_five)
    print(mat_joint)



"""pyfan generate and clean pdf files from folder
The :mod:`pyfan.util.pdf_pdfgen` generates pdf files from tex files.

Gather all tex files from a folder, allow for exclusion strings. Generate PDFs
from the tex files. And then clean up extraneous PDF outputs.

Includes method :func:`ff_pdf_gen_clean`.
"""

import subprocess
import os
from pathlib import Path


def ff_pdf_gen_clean(ls_spt_srh=None,
                     spt_out='C:/Users/fan/Documents/Dropbox (UH-ECON)/' +
                             'Project Emily Minority Survey/' +
                             'EthLang/reg_lang_abi_cls_mino/',
                     spn_pdf_exe='C:/texlive/2019/bin/win32/xelatex.exe',
                     ls_st_contain=None,
                     ls_st_ignore=None,
                     bl_recursive=False,
                     bl_clean=True,
                     ls_suf_clean=None):
    """Generate pdf files from latex files in various folders.

    This file serves important paper generation function. It compiles multiple
    files satisfying string search requirements or exclusion conditions in
    multiple folders, and saves resulting pdf outputs in one folder. This allows
    for easy testing and management of mutiple pdf/latex files for the same project.
    Suppose there is a longer version of a paper, a shorter version, and an
    appendix file. We want to regularly test the compilations of all files,
    otherwise, as we work on one of the files, perhaps we some something in the
    some shared files that lead to other files breaking without knowing.

    This should be run for all outward facing pdf/tex files for a project regularly
    in order to check if all files still compile.

    By brining resulting outputs to a single folder, this makes it easier to see
    all paper and project relevant outputs. Additionally, this cleans up all
    pdf generated extraneous files once we have pdf itself, saving pdf compile
    folder clutter.

    Parameters
    ----------
    ls_spt_srh : :obj:`list` of :obj:`str`
        A list of strings of the path in which to search for tex files. They should
        be all on the same path. If `bl_recursive` is true, then this searchs in
        all subfolders.
    spt_out : str
        The Path to store outputs. All PDFs stored under single directory. This
        path must be directly on the same path as `ls_spt_srh', can be higher up
        on the same tree, but not on a different branch.
    spn_pdf_exe: str
        The path to the pdflatex or alternative exe file
    ls_st_contain: :obj:`list` of :obj:`str`
        a list of strings the found names must contain one of these search words, not all, just one of.
    ls_st_ignore : :obj:`list` of :obj:`str`
        a list of string file names to ignore
    bl_recursive : bool
        Whether to search for all tex files within subfolders
    bl_clean : bool
        To clean up after file generation
    ls_suf_clean: :obj:`list` of :obj:`str`
        list of

    Returns
    -------
    dict
        A list of string pdf file names outputed,

    """

    # Set mutables
    if ls_spt_srh is None:
        ls_spt_srh = ['C:/Users/fan/Documents/Dropbox (UH-ECON)/' +
                      'Project Emily Minority Survey/' +
                      'EthLang/reg_lang_abi_cls_mino/']
    if ls_st_contain is None:
        ls_st_contain = None
        # ls_st_contain = ['_yg.']
    if ls_st_ignore is None:
        ls_st_ignore = ['all.tex']
    if ls_suf_clean is None:
        ls_suf_clean = ['.aux', '.log', '.out', '.bbl',
                        '.blg', '-blx.bib', '.fff', '.run.xml']

    # Search for tex files in folders
    if bl_recursive:
        # use rglob
        ls_spn_found = [spn_file
                        for spt_srh in ls_spt_srh
                        for spn_file in Path(spt_srh).rglob('*.tex')]
    else:
        # use glob
        ls_spn_found = [spn_file
                        for spt_srh in ls_spt_srh
                        for spn_file in Path(spt_srh).glob('*.tex')]

    # collect tex and pdf outputs
    dc_tex_pdf = {}
    # Generate pdf from found tex files
    for spn_found in ls_spn_found:
        # convert WindosPath to String
        spn_found = str(spn_found)

        # search through ignore list
        if sum([st_ignore in spn_found
                for st_ignore in ls_st_ignore]):
            pass
        else:

            if (ls_st_contain is None) or \
                    (sum([st_contain in spn_found
                          for st_contain in ls_st_contain]) > 0):

                # change to local directory
                os.chdir(os.path.split(spn_found)[0])

                # generate pdf
                subprocess.call([spn_pdf_exe, '-output-directory',
                                 spt_out, spn_found], shell=False)

                # collect results
                snm_file = Path(spn_found).stem
                dc_tex_pdf[spn_found] = spt_out + snm_file + '.pdf'

                # path no suffix
                if bl_clean:
                    path_no_suffix = os.path.splitext(spn_found)[0]
                    ls_st_remove_suffix = ls_suf_clean
                    for st_suffix in ls_st_remove_suffix:

                        # in tex directory
                        srn_cur_file = path_no_suffix + st_suffix
                        if os.path.isfile(srn_cur_file):
                            os.remove(srn_cur_file)

                        # in pdf output directory
                        srn_cur_file = spt_out + snm_file + st_suffix
                        if os.path.isfile(srn_cur_file):
                            os.remove(srn_cur_file)

    return dc_tex_pdf



"""
The :mod:`pyfan.gen.rand.randgrid` generate a grid with randomly spaced grid
points.

.. math:: x \\sim N\\left(\\mu, \\sigma\\right)

Includes method :func:`ar_draw_random_normal`.
"""

import numpy as np
from scipy.stats import norm


def ar_draw_random_normal(fl_mu, fl_sd, it_draws,
                          it_seed=None, it_draw_type=0,
                          fl_lower_sd=-3, fl_higher_sd=None):
    """Draw a Vector of Possibly Sorted and Bounded Normal Shocks

    Parameters
    ----------
    fl_mu, fl_sd : `float`
        The mean and standard deviation of the normal process
    it_draws: `int`
        Number of Draws
    it_seed: `int`, optional
        External random seed externally. Default is 123.
    it_draw_type: `int`, optional
        Indicates which type of normal draws to make. 0 sorted normal draws
        cut off at bounds. 1 equi-quantile unequal distance points;
        2 normal draws unsorted.
    fl_lower_sd, fl_higher_sd : `float`
        Impose lower and upper bounds (in sd units) on shock draws. The normal
        distribution does not have lower or upper bounds.

    Returns
    -------
    numpy.array of shape (1, it_draws)
        A vector of sorted or unsorted random grid points, or equi-quantile
        points.

    Notes
    -----
        This method requires a dataset of equal-sized time series

    Examples
    --------
    >>> fl_mu = 0
    >>> fl_sd = 1
    >>> it_draws = 5
    >>> it_seed = 123
    >>> fl_lower_sd = -1
    >>> fl_higher_sd = 0.8
    >>> it_draw_type = 0
    >>> ar_draw_random_normal(fl_mu, fl_sd, it_draws,
    ...                       it_seed, it_draw_type,
    ...                       fl_lower_sd, fl_higher_sd)
    [-1.          0.8         0.2829785 - 1. - 0.57860025]

    >>> it_draw_type = 1
    >>> ar_draw_random_normal(fl_mu, fl_sd, it_draws,
    ...                       it_seed, it_draw_type,
    ...                       fl_lower_sd, fl_higher_sd)
    [-1. - 0.47883617 - 0.06672597  0.3338994   0.8]

    >>> it_draw_type = 2
    >>> ar_draw_random_normal(fl_mu, fl_sd, it_draws,
    ...                       it_seed, it_draw_type,
    ...                       fl_lower_sd, fl_higher_sd)
    [-1. - 1. - 0.57860025  0.2829785   0.8]
    """

    # set seed
    if it_seed is None:
        it_seed = 123
    np.random.seed(it_seed)

    if (fl_higher_sd is None) and (fl_lower_sd is not None):
        fl_higher_sd = -fl_lower_sd

    # randomly drawing and then sorting
    if it_draw_type == 0 or it_draw_type == 2:
        ar_shock_draws = np.random.normal(0, 1, it_draws)
        if it_draw_type == 2:
            ar_shock_draws = np.sort(ar_shock_draws)
        if fl_lower_sd is not None:
            ar_shock_draws[ar_shock_draws < fl_lower_sd] = fl_lower_sd
        if fl_higher_sd is not None:
            ar_shock_draws[ar_shock_draws > fl_higher_sd] = fl_higher_sd
        ar_shock_draws = fl_mu + fl_sd * ar_shock_draws

    # Equi-quantile z points (but unequal x-distance)
    if it_draw_type == 1:
        if fl_lower_sd is None:
            fl_lower_sd = 0.0000001
        if fl_higher_sd is None:
            fl_higher_sd = 0.9999999
        lower_q = norm.cdf(fl_lower_sd)
        higher_q = norm.cdf(fl_higher_sd)
        ar_e_quantiles = np.linspace(lower_q, higher_q, it_draws)
        ar_shock_draws = norm.ppf(ar_e_quantiles, loc=fl_mu, scale=fl_sd)

    return ar_shock_draws



'''
Created on May 24, 2018

@author: fan

To have a better grid denser at the beginning
'''

import time as time
import numpy as np

from numba import jit

import logging

logger = logging.getLogger(__name__)


# @vectorize([float64(float64, float64, float64, float64, float64, float64, float64, float64)])
def grid_to_geom_short(choice_grid, choice_grid_max, choice_grid_min,
                       start, stop, num, geom_ratio, a):
    scaler = (choice_grid_max - choice_grid_min) / (stop - start)
    __, displacement, multiplier, a, b = gen_geom_grid(start, stop, num, geom_ratio, a)

    return grid_to_geom_short_core(choice_grid, a, scaler, displacement, multiplier, geom_ratio)


@jit(nopython=True, parallel=True)
def grid_to_geom_short_core(choice_grid, a, scaler, displacement, multiplier, geom_ratio):
    #     choice_grid_geom = ((choice_grid/scaler) + displacement)/multiplier
    #     but a is 1
    #     choice_grid_geom_base = (np.log(choice_grid_geom/a))/np.log(geom_ratio)

    return np.log((((choice_grid / scaler) + displacement) / multiplier) / a) / np.log(geom_ratio)


# @njit
def grid_to_geom(choice_grid, choice_grid_max, choice_grid_min,
                 start, stop, num, geom_ratio, a):
    """
    the code now is under the assumption that initial start and end were 0 and 1 
    
    Given geom_grid results, how do we go back to actual data grid. 
    So for interpolation. 
    interpolate not on actual K and B scales, but on any even grid, as long
    as the grid count is right. 
    
    interp_K_grid = np.linspace(0,1,n)
    
    but then there is a vector of actual choices kn_vec, how to map kn_vec to interp_K_grid?
    
    Parameters
    ----------
    choice_grid:
        this is the choice grid, on the actual choice scale
    start: float
        from gen_geom_grid
    stop: float
        from gen_geom_grid
    num: int
        from gen_geom_grid
    geom_ratio: float
        from gen_geom_grid        
    """
    #     logger.debug('enter grid_to_geom')

    '''
    0. Choice Grid Rescaling
    '''
    scaler = (choice_grid_max - choice_grid_min) / (stop - start)

    '''
    A. Reverse engineer from vector to geom scale
    '''
    #     startTime = time.time()
    __, displacement, multiplier, a, b = gen_geom_grid(start, stop, num, geom_ratio, a)
    #     logger.debug('displacement:%s', displacement)
    #     logger.debug('multiplier:%s', multiplier)
    #     logger.debug('a:%s', a)
    #     logger.debug('b:%s', b)
    #     t = time.time() - startTime
    #     print('Step aa:', t)

    '''choice_grid_geom is now between a and b'''
    #     startTime = time.time()
    choice_grid_geom = ((choice_grid / scaler) + displacement) / multiplier
    #     logger.debug('choice_grid:\n%s', choice_grid)
    #     logger.debug('choice_grid_geom:\n%s', choice_grid_geom)
    t = time.time() - startTime
    print('Step aaa:', t)

    '''
    B.
        a <= choice_grid_geom = a*(geom_ratio)^{x} <= b
        
        solve for x
        
        log(choice_grid_geom) = log(a) + x*log(geom_ratio)
        x = (log(choice_grid_geom) - log(a))/log(geom_ratio)
         
    '''
    startTime = time.time()
    choice_grid_geom_base = (np.log(choice_grid_geom / a)) / np.log(geom_ratio)
    #     choice_grid_geom_base = (np.log(choice_grid_geom))/np.log(geom_ratio)
    #     logger.debug('choice_grid_geom_base:\n%s', choice_grid_geom_base)
    t = time.time() - startTime
    print('Step bb:', t)

    return choice_grid_geom_base


# @njit
def gen_geom_grid(start, stop, num, geom_ratio, a):
    """    
    Specify geom_ratio, the z below:
        a*z^0=a
        a*z^1
        a*z^2
        ...
        ...
        a*z^49=b
    Then generate the grid points that is consistent with the geom_ratio
    
    Parameters
    ----------
    start: float
        same as in linspace
    stop: float
        same as in linspace
    num: int
        same as in linspace
    geom_ratio: float
        z value below kind of except for rescaling
    """

    '''
    A. Start with a and b
    '''
    #     a = 1
    b = a * geom_ratio ** (num - 1)

    geom_base = a * (geom_ratio) ** np.arange(num)
    #     geom_base2 = np.geomspace(a, b, num)
    #     logger.debug('geom_base:\n%s', geom_base)

    '''
    B. Rescaling
    '''
    multiplier = ((stop - start) / (b - a))
    geom_base_scaled = geom_base * multiplier
    #     logger.debug('geom_base_scaled:\n%s', geom_base_scaled)

    displacement = (np.min(geom_base_scaled) - start)
    geom_base_scaled = geom_base_scaled - displacement
    #     logger.debug('geom_base_scaled:\n%s', geom_base_scaled)

    #     logger.debug('geom_base_scaled diff:\n%s', np.diff(geom_base_scaled))

    return geom_base_scaled, displacement, multiplier, a, b


def tester(a=1, b=51, max_power=49):
    """
    1. 1 to 51, geomspace 
    """
    list_geom_1t51 = np.geomspace(a, b, max_power + 1)
    print('list_geom_1t51:', list_geom_1t51)

    """
    2. what is the list above, what does it mean?
    the point is to start from a=start, end at b=end, find:
    
    a*z^0=a
    a*z^1
    a*z^2
    ...
    ...
    a*z^49=b
    
    50 numbers like above. 
    a is determined by start of geomspace
    b is determined by: 
    
    """
    z = (b / a) ** (1 / max_power)
    print('z:', z)
    list_geom_fan = a * z ** np.linspace(0, max_power, max_power + 1)
    print('list_geom_fan:', list_geom_fan)

    return list_geom_fan


def tester_plus1(a=0, b=50, max_power=49, adjust=1):
    """
    to accomndate zero,  
    """
    list_geom_1t51 = np.geomspace(a + adjust, b + adjust, max_power + 1)
    list_geom_1t51 = list_geom_1t51 - 1
    print('list_geom_1t51:', list_geom_1t51)

    """
    2. what is the list above, what does it mean?
    the point is to start from a=start, end at b=end, find:
    
    a*z^0=a
    a*z^1
    a*z^2
    ...
    ...
    a*z^49=b
    
    50 numbers like above. 
    a is determined by start of geomspace
    b is determined by: 
    
    """
    z = ((b + adjust) / (a + adjust)) ** (1 / max_power)
    print('z:', z)
    list_geom_fan = (a + adjust) * z ** np.linspace(0, max_power, max_power + 1) - 1
    print('list_geom_fan:', list_geom_fan)

    """
    3. So suppose I have list_geom now, how to I take it back to geomspace?
    """
    lencount = len(list_geom_1t51)
    equi_space = np.linspace(1, lencount, lencount)
    print('equi_space:', equi_space)


if __name__ == "__main__":
    FORMAT = '%(filename)s - %(funcName)s - %(lineno)d -  %(asctime)s - %(levelname)s %(message)s'
    np.set_printoptions(precision=4, linewidth=100, suppress=True, threshold=np.nan)
    np.set_printoptions(precision=3, linewidth=100, suppress=True, threshold=3000)
    logging.basicConfig(level=logging.DEBUG, format=FORMAT)

    tester(a=1, b=2, max_power=49)
    tester(a=1, b=3, max_power=49)
    tester(a=1, b=4, max_power=49)
    print((tester(a=1, b=5, max_power=49) - 1) / 4)
    print('')
    print('')
    print('')
    tester(a=1, b=51, max_power=49)
    tester(a=0.1, b=51, max_power=10)
    tester(a=100, b=200, max_power=3)

    tester_plus1()

    print('')
    print('')
    print('')

    start = 0
    stop = 1
    num = 11
    geom_ratio = 1.2
    gen_geom_grid(start, stop, num, geom_ratio, 1)

    start = 10
    stop = 20
    num = 11
    geom_ratio = 1.00000001
    gen_geom_grid(start, stop, num, geom_ratio, 1)

    start = -10
    stop = 20
    num = 11
    geom_ratio = 1.1
    gen_geom_grid(start, stop, num, geom_ratio, 1)

    start = -3.5
    stop = -3.1
    num = 3
    geom_ratio = 1.1
    gen_geom_grid(start, stop, num, geom_ratio, 1)

    start = 0
    stop = 1
    num = 50
    geom_ratio = 1.03
    geom_base_scaled, __, __, __, __, = gen_geom_grid(start, stop, num, geom_ratio, 1)
    choice_grid_max = stop
    choice_grid_min = start
    grid_to_geom(geom_base_scaled, choice_grid_max, choice_grid_min,
                 start, stop, num, geom_ratio, 1)

    geom_base_scaled = np.linspace(0, 30, 22)
    choice_grid_max = 30
    choice_grid_min = 0
    grid_to_geom(geom_base_scaled, choice_grid_max, choice_grid_min,
                 start, stop, num, geom_ratio, 1)



'''
Created on Aug 14, 2018

@author: fan

Combine Files Together
'''

import logging
import glob as glob
import pandas as pd
import os.path

import pyfan.panda.inout.readexport as readexport

logger = logging.getLogger(__name__)


def search_combine(search_directory=None,
                   file_search_str=None,
                   save_file_name=None,
                   save_panda=True,
                   return_current=False):
    """
    Estimation saves csv files in different folders. Each folder has different starting value. 
    Different estimation method. 
    
    Gather results together, create single csv file. To find which parameters lead to smallest objective. 
    
    Parameters
    ----------
    return_current: boolean
        return_current if true, do not save file, return current file
        
    
    Examples
    --------
    import panda.io.combine as pd_combine
    all_esti_df = pd_combine.search_combine(search_directory = None,
                                            file_search_str = None, 
                                            save_file_name = None)
    
    """

    '''
    A. Piece together folder and search string
    '''
    if (search_directory is None):
        d_root = 'C:/Users/fan/Documents/Dropbox (UH-ECON)/Project Dissertation EC2/'
        d_submain = 'thaijmp201808j7itgesti/esti/'
        param_folder_name = 'c_20180814_list_all_params'
        search_directory = d_root + d_submain + param_folder_name + '/'
    if (file_search_str is None):
        file_search_str = '*/*.csv'

    '''
    B. Generate folder_name + file_name, check if file exists
    '''
    save_directory = search_directory
    if (save_file_name is None):
        file_name = param_folder_name + '.csv'
    else:
        if (save_file_name.endswith('.csv')):
            file_name = save_file_name
        else:
            file_name = save_file_name + '.csv'

    save_directory_file_name = save_directory + file_name

    file_exists = False
    if (os.path.isfile(save_directory_file_name)):
        file_exists = True

    '''
    B. Return existing file and exit
    '''
    if (return_current and file_exists):
        all_esti_df = readexport.read_csv(save_directory_file_name)
    else:
        '''
        C. Get files satisfying search condition
        '''
        file_list = glob.glob(search_directory + file_search_str)

        method_fast = True

        if (method_fast == True):
            '''
            This is significantly faster than the method below: 
                - the pd.concat function is only invoked once
            '''
            total_count = 0
            pd_list = []
            for ctr, file in enumerate(file_list):
                total_count = total_count + 1
                pd_list.append(pd.read_csv(file))
                if (ctr % 10 == 0):
                    logger.critical('pd_list file current %s of file total %s', total_count, len(file_list))
                    logger.critical('pd_list list length:%s', len(pd_list))

            all_esti_df = pd.concat(pd_list, ignore_index=True)

            if (all_esti_df is not None):
                logger.critical('search_combine total file count:%s', total_count)
                logger.critical('search_combine total row count:%s', all_esti_df.shape[0])

                if (save_panda):
                    all_esti_df.to_csv(save_directory + file_name, header=True, index=False)

        else:
            '''
            D. Import as panda from each file and then join panda file together
            '''
            total_count = 0
            all_esti_df = None
            for ctr, file in enumerate(file_list):
                total_count = total_count + 1
                df_cur = pd.read_csv(file)
                if (ctr == 0):
                    all_esti_df = df_cur
                else:
                    all_esti_df = pd.concat([all_esti_df, df_cur], ignore_index=True)

                if (ctr % 10 == 0):
                    logger.critical('search_combine file current %s of file total %s', total_count, len(file_list))
                    logger.critical('search_combine row count:%s', all_esti_df.shape[0])

            if (all_esti_df is not None):
                logger.critical('search_combine total file count:%s', total_count)
                logger.critical('search_combine total row count:%s', all_esti_df.shape[0])

                '''
                E. Save Results in single file under main folder
                '''
                if (save_panda):
                    all_esti_df.to_csv(save_directory + file_name, header=True, index=False)

    return all_esti_df


if __name__ == "__main__":
    search_combine()



'''
Created on Dec 2, 2017

@author: fan
'''

import numpy as np

def scalar_to_2darray(x, check_first=True):
    input_is_scalar = False
    if (check_first is True):
        if (np.isscalar(x)):
            input_is_scalar = True
        else:
            input_is_scalar = False

    if (input_is_scalar is True):
        x = np.asarray([[x]])
    else:
        pass

    return x

def scalar_to_array(x, check_first=True):
    input_is_scalar = False
    if (check_first is True):
        if (np.isscalar(x)):
            input_is_scalar = True
        else:
            input_is_scalar = False

    if (input_is_scalar is True):
        x = np.asarray(x)
        x = x[None]
    else:
        pass

    return x


def zero_ndims(ndims_var):
    """
    Parameters
    ----------
    ndims_var: array
        the dimension of this array to be duplicated

    """
    zero_array = 0
    if (np.isscalar(ndims_var)):
        zero_array = 0
    else:
        ndims = ndims_var.ndim
        if (ndims==1):
            zero_array = np.zeros(1)
        if (ndims==2):
            zero_array = np.zeros((1,1))
        if (ndims==3):
            zero_array = np.zeros((1,1,1))

    return zero_array


# # https://stackoverflow.com/questions/29318459/python-function-that-handles-scalar-or-arrays
# def func_for_scalars_or_vectors(x):
#     x = np.asarray(x)
#     scalar_input = False
#     if x.ndim == 0:
#         x = x[None]  # Makes x 1D
#         scalar_input = True
#
#     # The magic happens here
#
#     if scalar_input:
#         return np.squeeze(ret)
#     return ret

if __name__ == '__main__':

    print(scalar_to_array(1.0).shape)
    print(scalar_to_2darray(1.0).shape)

    print(scalar_to_2darray(np.array([1,2,3,4])).shape)



'''
Created on Aug 9, 2018

@author: fan
'''
import ast
import logging

import pandas as pd
from pandas.io.json import json_normalize

import pyfan.amto.json.json as support_json
import pyfan.panda.categorical.strsvarskeys as strvarskeys
import pyfan.util.inout.exportpanda as exportpanda

logger = logging.getLogger(__name__)
import shutil


def unflatten_denormalize(dictionary=None, dump_print=False):
    """https://stackoverflow.com/questions/6037503/python-unflatten-dict
    json_normalize results in dict that has dots for nests
    revert back
    
    Examples
    --------
    import panda.io.readexport as readexport
    dictionary = {}
    nested_dict = readexport.unflatten_denormalize(dictionary)
    """
    if (dictionary is None):
        dictionary = \
            {"esti_obj.main_allperiods_obj": 2161.704787809504,
             "data_model": "model",
             "P_jp_j.6.105": 0.35338853785616764,
             "P_jp_j.6.3": 0.08513942723795298,
             "data_param.A": -0.03125,
             "data_param.Region": 0,
             "data_param.Region_set": "[0, 1]",
             "data_param.Year": 0,
             "data_param.Year_set": "[0, 1]",
             "data_param.std_A": 0,
             "esti_obj.main_obj": 1321.7653586006118,
             "esti_obj.subsets_main.agg_prob_for": 6.551240600950623,
             "esti_obj.subsets_main.agg_prob_none": 0.01444353749392462,
             "esti_obj.subsets_main.equi_BI": 1246.3358962825505,
             "support_arg.workers": 1,
             "title": "kappa_ne0209=0.51150.2955:(INF BORR+SAVE)+(0MIN,0FC)+(LOG,Angeletos)BNF_BORR_P-0.500;BNF_SAVE_P-0.300;BNI_BORR_P-0.100;BNI_LEND_P-2.000;R_FORMAL_BORR-1.061;R_FORMAL_SAVE-1.011;R_INFORM_BORR-1.279;R_INFORM_SAVE-1.279;kappa-0.296;kappa_ne0209-0.296;kappa_ne9901-0.511 (g:basic, e:zeroFE, e:Angeletos, e:0,1,102,3,104,105,6, i:forgegeom)",
             "y_opti_grid_allJ_agg_var": 5.9793820314678605,
             "interpolant.interp_type": "['forgegeom']"
             }

    nested_dict = dict()
    for key, value in dictionary.items():
        parts = key.split(".")
        d = nested_dict
        for part in parts[:-1]:
            if part not in d:
                d[part] = dict()
            d = d[part]
        if (isinstance(value, str) and value.startswith('[')):
            d[parts[-1]] = ast.literal_eval(value)
        else:
            d[parts[-1]] = value

    if (dump_print):
        support_json.jdump(nested_dict, 'resultDict',
                           logger=logger.debug, print_here=True)

    return nested_dict


def read_stata(file_directory_name):
    df = pd.read_stata(file_directory_name)

    return df


def read_file(file_key='thai_data_z'):
    data_directory = strvarskeys.main_data_directory()

    file_names_dict = strvarskeys.file_names()
    data_z_strs = file_names_dict[file_key]

    z_df = read_file_main(file_name=data_z_strs['filename'],
                          format_suffix=data_z_strs['format_suffix'],
                          directory=data_directory,
                          data_format=data_z_strs['format'])

    return z_df


def read_csv(csv_file_folder):
    """
    directory = 'C:/Users/fan/Documents/Dropbox (UH-ECON)/Project Dissertation/model_test/test_solu/'
    param_combo = '20180701_basicJ7_basic'
    file_name = 'solu_'+param_combo+'.csv'
    csv_file_folder = directory + '/' + file_name
    solu_opti_pd = proj_sys_sup.read_csv(csv_file_folder)
    """
    return pd.read_csv(csv_file_folder)


def read_file_main(file_name, format_suffix, directory, data_format='stata', show_columns=True):
    if (data_format == 'stata'):
        if (directory.endswith('/')):
            file_full = directory + file_name + format_suffix
        else:
            file_full = directory + '//' + file_name + format_suffix
        panda_df = read_stata(file_full)

    if (show_columns):
        support_json.jdump(list(panda_df.columns.values),
                           'list(z_df.columns.values)',
                           logger=logger.info,
                           print_here=True)

    panda_df.file_name = file_name
    panda_df.directory = directory
    panda_df.data_format = data_format
    panda_df.vars_c = strvarskeys.var_names(file_name=file_name, out_type='coln')
    panda_df.vars_d = strvarskeys.var_names(file_name=file_name, out_type='desc')

    return panda_df


def dict_to_panda(list_of_dict, save_file_directory=None):
    panda_df = json_normalize(list_of_dict)
    logger.info('panda_df\n:%s', panda_df)

    if (save_file_directory is not None):
        exportpanda.saveCSV(saveFileDirectory=save_file_directory,
                            dataToSave=panda_df,
                            header='',
                            rowindex=False,
                            is_panda=True,
                            replace=True,
                            export=True)

    return panda_df


if __name__ == "__main__":
    unflatten_denormalize(dictionary=None, dump_print=True)



'''
Created on Mar 28, 2018

@author: fan
'''

import logging

logger = logging.getLogger(__name__)

import pandas as pd
import os as os
import json

import pyfan.amto.json.json as support_json


def export_history_csv(store, store_map, filepath='', filename='', export=True):
    logger.info('filename:\n%s', filename)
    logger.info('filepath:\n%s', filepath)
    logger.info('store.shape:\n%s', store.shape)

    saveFileDirectory = filepath + filename + '.csv'

    return saveCSV(saveFileDirectory, store, list(store_map.keys()), is_panda=False, export=export)


def saveJSON(saveFileDirectory, data, replace):
    open_str = 'w'
    if (replace):
        open_str = 'w'
    else:
        open_str = 'a'

    with open(saveFileDirectory, open_str) as outfile:
        json_out = json.dump(data, outfile, default=support_json.json_serial)

    return json_out


def saveCSV(saveFileDirectory, dataToSave, header='', rowindex=False,
            is_panda=True, replace=True, export=True):
    'if get permissiond denied error, because possible saveFileDirectory did not have .csv end'

    export_index = False
    if (is_panda == True):
        df = dataToSave
    else:
        df = pd.DataFrame(dataToSave)
        df.columns = header
        if (rowindex is not False):
            df.index = rowindex
            export_index = True

    if (export):
        if (replace == False):
            'check if file already exists'
            if (os.path.exists(saveFileDirectory)):
                df_existing = pd.read_csv(saveFileDirectory)
                df_existing = df_existing.append(df)
                df = df_existing
                df = df.drop_duplicates()

        df.to_csv(saveFileDirectory, header=True, index=export_index)

    return df



'''
Created on Sep 23, 2018

@author: fan
'''


import logging
import pyfan.amto.json.json as support_json

import statsmodels.api as sm
import statsmodels.formula.api as smf
import numpy as np
import pandas
from sklearn.linear_model import Ridge
from sklearn.preprocessing import PolynomialFeatures

logger = logging.getLogger(__name__)


def ols_formula(df, dependent_var, *excluded_cols):
    '''
    Generates the R style formula for statsmodels (patsy) given
    the dataframe, dependent variable and optional excluded columns
    as strings
    '''
    df_columns = list(df.columns.values)
    df_columns.remove(dependent_var)
    for col in excluded_cols:
        df_columns.remove(col)
        
    return dependent_var + ' ~ ' + ' + '.join(df_columns)


def tester():
#     df = sm.datasets.get_rdataset("Guerry", "HistData").data
#     print(df.columns)
#     df = df[['Lottery', 'Literacy', 'Wealth', 'Region']].dropna()

    folder = 'C:/Users/fan/Documents/Dropbox (UH-ECON)/Project Dissertation EC2/thaijmp201809j8vara/esti/c_20180918_ITG_list_tall_mlt_ce1a2/'
#     file_base = 'c_20180918_ITG_list_tall_mlt_ce1a2'
    file_base = 'c_20180918_ITG_list_tall_mlt_ce1a2_modelsimu_regress'
    file = file_base + '.csv'
    
    file_path = folder + file
    df_use = proj_sys_sup.read_csv(csv_file_folder=file_path)
    df_use_columns = df_use.columns.tolist()
    support_json.jdump((df_use.columns).tolist(), 'df_use.columns', logger=logger.warning)
    
    
    poly = PolynomialFeatures(2)
#     df_inter = poly.fit_transform(df_use[['esti_param.BNF_SAVE_P_ce9901', 'esti_param.BNF_BORR_P_ce0209']])
    
    columns_rhs = []
    for columns in df_use.columns:
        if ('esti_obj' in columns):
            pass
        elif ('_ce0209' in columns):
            pass
        else:
            columns_rhs.append(columns)
    
    support_json.jdump((columns_rhs), 'columns_rhs', logger=logger.warning)
    support_json.jdump((len(columns_rhs)), 'len(columns_rhs)', logger=logger.warning)        
    
    '''
    if 13 + 1 (constant) rhs, this should be 105 terms for poly(2)
    half matrix = (14*14 - 14)/2 + 14
    '''
    
    X = poly.fit_transform(df_use[columns_rhs])
    y = df_use['esti_obj.main_allperiods_obj']
    est = sm.OLS(y, X).fit()
    
    
    print(est.summary())
    
    
    
    
#     mod = smf.ols(formula='Lottery ~ .', data=df)
#     res = mod.fit()
#     print(res.summary())
    
#     support_json.jdump(df.head().values().tolist(), 'df.head()', logger=logger.warning)
    
    
if __name__ == '__main__':
    
    FORMAT = '%(filename)s - %(funcName)s -\n %(asctime)s - %(levelname)s  - %(message)s'
    FORMAT = '%(funcName)s - %(asctime)s - %(levelname)s  - %(message)s'
    FORMAT = '%(filename)s - %(funcName)s - %(lineno)d -  %(asctime)s - %(levelname)s %(message)s'
    logging.basicConfig(level=logging.DEBUG, format=FORMAT)
    tester()
    
    
    
    
    


'''
Created on Dec 4, 2017

@author: fan
'''

import logging
import numpy as np

logger = logging.getLogger(__name__)


class UtilityMultiNomial():
    """
        each_j_indirect_utility:
            N by J matrix
            
        N is the number of individuals (unique states)
        J is the number of choices
        
        N might be 0 
    """

    def __init__(self, scale_coef=1):
        """
        Paramters
        ---------
        scale_coef: scalar
            Usually this is not an important paramter, it is usually 1, but 
            based on my reading for nonlinear logit, need, could have a scale
            coefficient, could test this out, anyway, this is to be estimated
            potentially if the indirect utility component is nonlinear
            
        """
        '''
        Train Page 40 about scale parameter
        in linear in parameter logit, when rescaling linear parameters
        scale coefficient does not matter. But in nonlinear, they do. 
        '''
        self.scale_coef = scale_coef

    def prob_denominator(self, all_J_indirect_utility):
        '''
        if: 
                all_J_indirect_utility/self.scale_coef = -598.66/0.75
            then: 
                prob_denominator = exp(-598.66/0.75) = 0.0
            then: 
                sum(prob_denominator) = 0
            then:
                np.exp(all_J_indirect_utility/self.scale_coef)/prob_denominator_tile = INVALID
        so there must be some minimal level for the division here. in terms of scaling
        '''
        #         all_J_indirect_utility/self.scale_coef

        #         prob_denominator = np.sum(np.exp(all_J_indirect_utility/self.scale_coef), axis=1)
        prob_denominator = np.sum(np.exp(all_J_indirect_utility), axis=1)

        return prob_denominator

    #         np.exp(-598.66)
    def prob_j(self, all_J_indirect_utility, prob_denominator=None):
        if (prob_denominator is None):
            prob_denominator = self.prob_denominator(all_J_indirect_utility)

        choice_J_count = np.shape(all_J_indirect_utility)[1]
        logger.debug('choice_J_count:\n%s', choice_J_count)

        prob_denominator_tile = np.transpose(
            np.tile(prob_denominator, (choice_J_count, 1)))

        logger.debug('all_J_indirect_utility:\n%s', all_J_indirect_utility)
        logger.debug('self.scale_coef:\n%s', self.scale_coef)
        logger.debug('prob_denominator_tile:\n%s', prob_denominator_tile)

        #         each_j_prob = np.exp(all_J_indirect_utility/self.scale_coef)/prob_denominator_tile
        each_j_prob = np.exp(all_J_indirect_utility) / prob_denominator_tile

        logger.debug('each_j_prob:\n%s', each_j_prob)
        return each_j_prob

    def expected_u_integrate_allj(self, prob_denominator):
        '''
        'see Train discussion on consumer surplus and logit'
        'Need to check the reference that Train cites to make sure this integration applies in my case, should derive it myself'
        If one option has much higher utility, 
        and if variance is low, integrated utility is linear in this option, 
        in fact they are equal 
        '''
        optiV_Exp7 = np.log(prob_denominator)
        return optiV_Exp7

    def get_outputs(self, all_J_indirect_utility):
        all_J_indirect_utility = all_J_indirect_utility / self.scale_coef

        brk_low = -600
        gap = 100
        base = (brk_low - gap)
        too_low_idx = [(all_J_indirect_utility) <= brk_low]
        all_J_indirect_utility[too_low_idx] = base + (
                1 / (brk_low + 1 - all_J_indirect_utility[too_low_idx]) ** (1 / 4)) * 100

        brk_high = 600
        base = (brk_high + gap)
        too_high_idx = [(all_J_indirect_utility) >= brk_high]
        all_J_indirect_utility[too_high_idx] = base - (
                1 / (all_J_indirect_utility[too_high_idx] - brk_high + 1) ** (1 / 4)) * 100

        logger.debug('all_J_indirect_utility:\n%s', all_J_indirect_utility)
        prob_denominator = self.prob_denominator(all_J_indirect_utility)
        logger.debug('prob_denominator:\n%s', prob_denominator)
        each_j_prob = self.prob_j(all_J_indirect_utility, prob_denominator)
        optiV_Exp7 = self.expected_u_integrate_allj(prob_denominator)

        return each_j_prob, optiV_Exp7



'''
Created on Sep 24, 2013

@author: fan
'''
"""
ax1 = plt.subplot2grid((3,3), (0,0), colspan=3)
ax2 = plt.subplot2grid((3,3), (1,0), colspan=2)
ax3 = plt.subplot2grid((3,3), (1, 2), rowspan=2)
ax4 = plt.subplot2grid((3,3), (2, 0))
ax5 = plt.subplot2grid((3,3), (2, 1))
"""

import logging

logger = logging.getLogger(__name__)

import matplotlib

# matplotlib.use('Agg')

# matplotlib.use('qt5agg')
# import matplotlib.pyplot as pylab
import pylab as pylab
# import matplotlib.pyplot as plt
import numpy as np
from scipy.stats import gaussian_kde
# from matplotlib.mlab import griddata
import mpl_toolkits.mplot3d.axes3d as p3
import itertools
from matplotlib.patches import Polygon
from matplotlib import cm

import seaborn as sns


# ===============================================================================
# pylab.tight_layout(pad=0.4, w_pad=0.5, h_pad=1.0)
# pylab.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=None, hspace=None)
# ===============================================================================

def contourAnd3D(xData, yData, zData,
                 xLabStr, yLabStr, zLabStr,
                 graphTitleDisp, graphTitleSave,
                 savedpi=125, angleType=[1, [1, 2, 3]],
                 drawContour=False, draw3D=True,
                 draw3DSurf=False,
                 contourXres=100, contourYres=100,
                 s=20, alpha=0.6,
                 subplot=None, fig=None):
    if (fig == None):
        fig = pylab.figure()

    '''
    Contour
    '''
    if (drawContour == True):
        try:
            pylab.clf()
            if (subplot is not None):
                pylab.subplot(subplot['rows'], subplot['cols'], subplot['cur'])
            xN, yN, zN = grid(xData, yData, zData, resX=contourXres, resY=contourYres)
            pylab.contour(xN, yN, zN)
            pylab.xlabel(xLabStr)
            pylab.ylabel(yLabStr)
            pylab.title(graphTitleDisp)
            pylab.grid()
            # pylab.show()
            pylab.savefig(graphTitleSave + '_contour', dpi=savedpi, papertype='a1')
        except:
            pass

    '''
    3D
    '''
    if (draw3D == True):
        try:
            pylab.clf()
            if (subplot is not None):
                pylab.subplot(subplot['rows'], subplot['cols'], subplot['cur'])
            ax = p3.Axes3D(fig)
            ax.scatter3D(xData, yData, zData, marker='o', s=s, c=np.ravel(zData), alpha=alpha)
            tripleAngle3dSave(ax, graphTitleDisp, xLabStr, yLabStr, zLabStr, graphTitleSave + '_3D', savedpi=savedpi,
                              angleType=angleType)
        except:
            pass

    if (draw3DSurf == True):
        try:
            #             pylab.clf()
            if (subplot is not None):
                ax = fig.add_subplot(subplot['rows'], subplot['cols'], subplot['cur'], projection='3d')
            else:
                ax = p3.Axes3D(fig)
            ax.plot_surface(xData, yData, zData, cmap=cm.coolwarm, linewidth=0, antialiased=False)
            tripleAngle3dSave(ax, graphTitleDisp, xLabStr, yLabStr, zLabStr, graphTitleSave + '_3DS', savedpi=savedpi,
                              angleType=angleType)
        except:
            pass

    return fig


#     pylab.close(fig)

"""
angleType: [1, [1, 2, 3]]
    this menas
"""


def tripleAngle3dSave(ax, graphTitleDisp, xLabStr, yLabStr, zLabStr, graphTitleSave, savedpi=125,
                      angleType=[1, [1, 2, 3]]):
    ax.set_title(graphTitleDisp)
    # pylab.show()
    # for graphICur in [1, 2, 3]:
    for graphICur in angleType[1]:
        ax.set_xlabel(xLabStr)
        ax.set_ylabel(yLabStr)
        ax.set_zlabel(zLabStr)
        if (angleType[0] == 1):
            if (graphICur == 1):
                ax.view_init(elev=0, azim=-180)
                ax.set_xlabel('')
            if (graphICur == 2):
                ax.view_init(elev=0, azim=-90)
                ax.set_ylabel('')
            if (graphICur == 3):
                ax.view_init(elev=90, azim=-90)
                ax.set_zlabel('')
            if (graphICur == 4):
                ax.view_init(elev=-90, azim=0)
                ax.set_ylabel('')
            if (graphICur == 5):
                ax.view_init(elev=0, azim=-145)
                ax.set_ylabel('')
            if (graphICur == 6):
                ax.view_init(elev=22., azim=-115)
                ax.set_ylabel('')
        if (angleType[0] == 2):
            if (graphICur == 1):
                ax.view_init(elev=-90, azim=0)
            if (graphICur == 2):
                ax.view_init(elev=0, azim=-145)
        if (angleType[0] == 3):
            if (graphICur == 1):
                ax.view_init(elev=22., azim=-115)

        pylab.savefig(graphTitleSave + str(graphICur), dpi=savedpi, papertype='a1')

    # ===========================================================================
    #
    # angleColl_elevAndAzim_set1 = [[90,-90],[0,-180],[0,-90]]
    # angleColl_elevAndAzim_set2 = [[-90,0],[0,-145]]
    # angleColl_elevAndAzim = [angleColl_elevAndAzim_set1,angleColl_elevAndAzim_set2]
    #
    # if (angleColl_elevAndAzim[0] == 1):
    #     ax.set_title(graphTitleDisp)
    #     # pylab.show()
    #     #for graphICur in [1, 2, 3]:
    #     for graphICur in angleType[1]:
    #         ax.set_xlabel(xLabStr)
    #         ax.set_ylabel(yLabStr)
    #         ax.set_zlabel(zLabStr)
    #         (curElev,curAxzim) = angleColl_elevAndAzim[angleType[0]][graphICur]
    #         ax.view_init(elev=curElev, azim=curAxzim)
    #         if (graphICur == 1):
    #             ax.set_zlabel('')
    #         if (graphICur == 2):
    #             ax.set_xlabel('')
    #         if (graphICur == 3):
    #             ax.set_ylabel('')
    #
    #         pylab.savefig(graphTitleSave+str(graphICur), dpi=savedpi, papertype='a1')
    #
    # if (angleType[0] == 2):
    #     ax.view_init(elev=-90, azim=0)
    # ===========================================================================


'''
Essential function for turning scatter 3d plot into contour 3d plot
'''


def grid(x, y, z, resX=100, resY=100):
    "Convert 3 column data to matplotlib grid"
    xi = np.linspace(min(x), max(x), resX)
    yi = np.linspace(min(y), max(y), resY)
    Z = griddata(x, y, z, xi, yi)
    X, Y = np.meshgrid(xi, yi)
    return X, Y, Z


toGraphHere = False


def graph_emaxKCash_Value(soluSupObj, resources, k_vec, emaxValsCur, emaxChoicesCur, emaxChoiceOfMaxCollCur,
                          predictUtil):
    grapher = graphFunc()
    pylab.clf()

    toGraphHere = False
    if (toGraphHere == True):
        # fig=pylab.figure()
        xLabStr = 'Cash on Hand'
        yLabStr = 'Physical Capital'
        xData = resources
        yData = k_vec

        '''
        Resource and K State Space Ranges
        '''
        # ===================================================================
        # grapher = grpSup.graphFunc()
        # pylab.clf()
        # grapher.xyPlotMultiYOneX(yDataMat=resources, saveOrNot=True, showOrNot=False, graphType='hist',
        #                          saveDirectory=fobj.support_args['IO'],
        #                          saveFileName='ResStateHist'+str(fobj.support_args['cur_round']),
        #                          basicTitle='Resource Histogram', basicXLabel='Resource State Points', basicYLabel='Density')
        #
        # pylab.clf()
        # grapher.xyPlotMultiYOneX(yDataMat=k_vec, saveOrNot=True, showOrNot=False, graphType='hist',
        #                          saveDirectory=fobj.support_args['IO'],
        #                          saveFileName='KStateHist'+str(fobj.support_args['cur_round']),
        #                          basicTitle='K Histogram', basicXLabel='K State Points', basicYLabel='Density')
        # ===================================================================

        titleStrList = ['Utility', 'Consumption Choice', 'Kapital Choice', 'B Formal Choice', 'B Informal Choice']
        titleStringList = ['Informal Borrow', 'Informal Save/Lend', 'Formal Borrow', 'Formal Save',
                           'Formal and Informal Borrow', 'Formal Borrow and Informal Save/Lend', 'None']
        titleStringList = ['Inf Borr', 'Inf Save', 'For Borr', 'Form Save',
                           'FB+IB', 'FB+IS', 'None']

        drawHere2DGraph = True
        drawHere3DGraph = False

        """
        3D and Contour
        """
        curRound = str(soluSupObj.sa['cur_round'])
        if (int(curRound) % 1 == 0):

            """
            Look over Value and Choices
            """
            # for curCol in range(0,0):

            if (drawHere2DGraph == True):
                pylab.clf()
                fig, ((ax1, ax2, ax3), (ax4, ax5, ax6)) = pylab.subplots(2, 3)
                axisList = [ax1, ax2, ax3, ax4, ax5, ax6]

            for curCol in range(0, 5):
                pylab.sca(axisList[curCol])

                titlStr = titleStrList[curCol]
                labelArray_coll = [titlStr]
                noLabel = True

                if (titlStr == 'Utility'):
                    zData = emaxValsCur[:, 0]
                    angleType = [1, [1, 2, 3]]
                else:
                    zData = emaxChoicesCur[:, curCol - 1]
                    angleType = [3, [1]]

                if (drawHere2DGraph == True):

                    """
                    Draw Prediction Line
                    """
                    if (curCol == 0):
                        xDataSortedIdx = np.argsort(xData)
                        xDataSorted = xData[xDataSortedIdx]
                        predictUtilSorted = predictUtil[xDataSortedIdx]

                        grapher.xyPlotMultiYOneX(yDataMat=predictUtilSorted, xData=xDataSorted,
                                                 labelArray=labelArray_coll, noLabel=noLabel,
                                                 saveOrNot=False, graphType='plot')

                    """
                    Draw Choices and Value along resources, scatter, color showing K value
                    """
                    line45Deg = False
                    if (curCol == 1):
                        line45Deg = True

                    grapher.xyPlotMultiYOneX(yDataMat=zData, xData=xData, colorVar=yData,
                                             labelArray=labelArray_coll, noLabel=noLabel,
                                             saveOrNot=False, graphType='scatter', scattersize=1,
                                             labelLoc1t0=4, basicTitle=titlStr,
                                             basicXLabel='Cash on Hand', basicYLabel='Consumption Unit',
                                             line45Deg=line45Deg)

                if (drawHere3DGraph == True and curCol <= 4):
                    graphTitleDisp = titlStr
                    saveTitleFull = soluSupObj.sa['IO'] + 'EmaxApprox_VCKnBn_t' + str(curCol) + 'iter' + curRound
                    graphTitleSave = saveTitleFull
                    zLabStr = 'Choices'

                    contourAnd3D(xData, yData, zData, xLabStr, yLabStr, zLabStr,
                                 graphTitleDisp, graphTitleSave, angleType=angleType, drawContour=True, draw3D=True)

            if (drawHere2DGraph == True):
                grapher.savingFig(saveDirectory=soluSupObj.sa['IO'],
                                  saveFileName='EmaxApprox_VCKnBn_iter' + str(curRound),
                                  saveDPI=200, pylabUse=fig)

            """
            Probability of Choices Graphs along 3D
            """
            # for curCol in range(4,4,1):
            if (drawHere2DGraph == True):
                pylab.clf()
                fig, ((ax1, ax2, ax3, ax4), (ax5, ax6, ax7, ax8)) = pylab.subplots(2, 4)
                axisList = [ax1, ax2, ax3, ax4, ax5, ax6, ax7]

            maxProb = np.max(emaxChoiceOfMaxCollCur[:, 4:11])
            for curCol in range(4, 11, 1):
                pylab.sca(axisList[curCol - 4])

                xLabStr = 'Cash on Hand'
                yLabStr = 'Physical Capital'
                titlStr = 'Probability of ' + titleStringList[curCol - 4]

                zData = emaxChoiceOfMaxCollCur[:, curCol]

                if (drawHere3DGraph == True):
                    graphTitleDisp = titlStr
                    saveTitleFull = soluSupObj.sa['IO'] + 'EmaxApprox_Prob_t' + str(curCol) + 'iter' + curRound
                    graphTitleSave = saveTitleFull
                    zLabStr = 'Probability of Choice'

                    contourAnd3D(xData, yData, zData, xLabStr, yLabStr, zLabStr,
                                 graphTitleDisp, graphTitleSave, angleType=[3, [1]], drawContour=True, draw3D=True)

                if (drawHere2DGraph == True):
                    grapher.xyPlotMultiYOneX(yDataMat=zData, xData=xData, colorVar=yData,
                                             saveOrNot=False, graphType='scatter', scattersize=1,
                                             basicTitle=titleStringList[curCol - 4],
                                             basicXLabel='Cash on Hand', basicYLabel='Probability',
                                             ylim=[-0.05, maxProb], xlim=[0 - 5000, np.max(xData) + 5000])

            """
            sumProb Should be 1, but maybe not due to code issues, need to check here
            """
            pylab.sca(ax8)
            sumProb = np.sum((emaxChoiceOfMaxCollCur[:, 4:11]), axis=1)
            grapher.xyPlotMultiYOneX(yDataMat=sumProb, xData=xData,
                                     saveOrNot=False, graphType='scatter', scattersize=1, basicTitle='Sum Prob',
                                     basicXLabel='Cash on Hand', basicYLabel='Probability', ylim=[-0.05, 1.05],
                                     xlim=[0 - 5000, np.max(xData) + 5000])

            if (drawHere2DGraph == True):
                grapher.savingFig(saveDirectory=soluSupObj.sa['IO'],
                                  saveFileName='EmaxApprox_Prob_iter' + str(curRound),
                                  saveDPI=200, pylabUse=fig)


def OLSEmaxValAndChoicesGraphs(allDataY, allDataX,
                               saveFileSuffix='', yLabelNames=['Emax', 'Choice'],
                               xLabelNames=['Height', 'Weight', 'Income'],
                               saveDirectory='default', saveFileName='default'):
    # toGraphHere = False
    if (toGraphHere == True):

        loopCountY = len(yLabelNames)
        for graphI in range(0, loopCountY, 1):
            yData = allDataY[graphI]
            OLSEmaxGraphs(saveFileSuffix, yVal=yData, allDataX=allDataX,
                          yLabelName=yLabelNames[graphI], xLabelNames=xLabelNames,
                          saveDirectory=saveDirectory, saveFileName=saveFileName)


def OLSEmaxGraphs(saveFileSuffix,
                  yVal, allDataX,
                  saveDirectory='default', saveFileName='default', yLabelName='yLabelName',
                  xLabelNames=['Height', 'Weight', 'Income']):
    if (saveDirectory == 'default'):
        saveDirectory = 'C:\\Users\\fan\\Documents\\Dropbox\\Height_Production_Function\\Results--HD on HL WL--Linear--Protein and Calorie With Price Instruments\\ProduWithPref\\'

    saveFileName = saveFileName + '_' + str(yLabelName) + str(saveFileSuffix)
    if (saveFileName == 'default'):
        saveFileName = 'EmaxApprox'

    grapher = graphFunc()
    pylab.clf()

    loopCount = len(xLabelNames)
    for graphI in range(0, loopCount, 1):
        xData = allDataX[graphI]
        yDataMat = yVal
        pylab.subplot(2, loopCount, graphI + 1)
        basicTitle = xLabelNames[graphI] + ' and ' + yLabelName
        basicXLabel = xLabelNames[graphI]
        basicYLabel = yLabelName
        grapher.xyPlotMultiYOneX(yDataMat=yDataMat, xData=xData, saveOrNot=False, graphType='scatterregline',
                                 basicTitle=basicTitle, basicXLabel=basicXLabel, basicYLabel=basicYLabel, noLabel=True,
                                 scattersize=0.1)

        xData = allDataX[graphI]
        pylab.subplot(2, loopCount, graphI + 1 + loopCount)
        basicTitle = xLabelNames[graphI] + ' Histogram'
        basicXLabel = xLabelNames[graphI]
        basicYLabel = 'Frequency'
        grapher.xyPlotMultiYOneX(yDataMat=xData, saveOrNot=False, graphType='hist',
                                 basicTitle=basicTitle, basicXLabel=basicXLabel, basicYLabel=basicYLabel, noLabel=True,
                                 bins=30)

        pylab.tight_layout(pad=0.4, w_pad=0.5, h_pad=1.0)
        pylab.subplots_adjust(left=None, bottom=None, right=None, top=None, wspace=None, hspace=None)

    grapher.savingFig(saveDirectory=saveDirectory, saveFileName=saveFileName)


class graphFunc:
    points = 200
    xData = np.linspace(-1, 1, points)
    xData = np.random.normal(10, 2, points)
    xData = np.sort(xData)
    yData1 = np.random.normal(0, 1, points)
    yData2 = np.random.normal(0, 1, points) + 10
    yDataMat = np.column_stack((yData1, yData2))
    # ===============================================================================

    # Legend location
    # The location of the legend can be specified by the keyword argument loc, either by string or a integer number.
    # String    Number
    # upper right    1
    # upper left    2
    # lower left    3
    # lower right    4
    # right    5
    # center left    6
    # center right    7
    # lower center    8
    # upper center    9
    # center    10
    # ===============================================================================

    labelLoc1t0 = 'best'
    labelColCount = 1
    labelArray = ['line y1', 'line y2']
    basicTitle = 'Image  Name'
    basicXLabel = 'X Title Name'
    basicYLabel = 'Y Title Name'

    showOrNot = False

    # saveDirectory = 'C:/Users/fan/Documents/Dropbox/Programming/Sandbox/Graphs/'
    saveDirectory = 'C:/Users/fan/Pictures'
    saveFileName = 'temp.png'
    saveDPI = 125

    colorCounter = 0

    # yDataMat should be N by K, and xData is N by 1
    def __init__(self,
                 showOrNot=False,
                 saveDirectory=saveDirectory,
                 saveDPI=saveDPI
                 ):
        self.showOrNot = showOrNot
        self.saveDirectory = saveDirectory
        self.saveDPI = saveDPI
        self.colorCounter = 0

    def xyPlotMultiYOneX(self, xData=xData, yDataMat=yDataMat, colorVar=None,
                         labelArray=labelArray, noLabel=True,
                         basicTitle=basicTitle, basicXLabel=basicXLabel, basicYLabel=basicYLabel,
                         labelLoc1t0=labelLoc1t0, labelColCount=labelColCount,
                         line45Deg=False,
                         showOrNot=False, saveOrNot=True, graphType='plot',
                         saveDirectory=saveDirectory, saveFileName=saveFileName, saveDPI=1000, toScale=True,
                         pylabUse=None,
                         ylim=None, xlim=None,
                         sequential_color=False,
                         subplot=None,
                         clear_first=False,
                         **keywords):
        """Graph general

        yDataMat each column corresponds to x
        """
        # ===========================================================================
        #     import Support.GraphSupport as grhSup
        #     import pylab as pylab
        #     grapher = grhSup.graphFunc()
        #     pylab.clf()
        #
        #     xData = prot
        #     yDataMat = util_at_protrange
        #
        #     basicTitle = 'Utility at Choices Mean States'
        #     basicXLabel = 'Protein'
        #     basicYLabel = basicTitle
        #     grapher.xyPlotMultiYOneX(yDataMat=yDataMat,xData=xData,saveOrNot=False, graphType='plot',
        #                              basicTitle=basicTitle, basicXLabel=basicXLabel, basicYLabel=basicYLabel,noLabel=True)
        #
        #     grapher.savingFig(saveDirectory=self.support_args['IO'],saveFileName='graphMeanChoice_r'+str(self.support_args['cur_round']))
        # ===========================================================================

        # TESTER: grapher = graphFunc().xyPlotMultiYOneX()

        # ===============================================================================
        # TESTER 2:
        #
        # grapher = graphFunc()
        # pylab.subplot(2,1,1)
        # grapher.xyPlotMultiYOneX(saveOrNot=False)
        # pylab.subplot(2,1,2)
        # grapher.xyPlotMultiYOneX(saveOrNot=False)
        # grapher.savingFig()
        # pylab.clf()
        # grapher.xyPlotMultiYOneX(saveOrNot=True,saveFileName='temp2')
        # ===============================================================================

        if (subplot is not None):
            pylab.subplot(subplot['rows'], subplot['cols'], subplot['cur'])

        if (pylabUse == None):
            pylabUse = pylab

        try:
            pylab.setp(pylabUse.xaxis.get_majorticklabels(), rotation=70)
        except:
            pylab.xticks(rotation=45)

        if (clear_first is True):
            pylab.cla()
            pylab.clf()
            pylab.close()
            pylab.gcf().clear()

        if (graphType.lower() == "polygon"):
            colCnt = 1
        else:
            curDim = np.ndim(yDataMat)
            if (curDim > 1):
                rowCnt, colCnt = (yDataMat.shape)
            else:
                colCnt = 1

        colorSet = ['b', 'g', 'r', 'c', 'm', 'y', 'k']
        if (colCnt > 1 and sequential_color == True):
            colorSet = sns.color_palette("coolwarm", colCnt)
            # colorSet = sns.diverging_palette(255, 133, l=60, n=colCnt, center="dark")
            sns.set_style("whitegrid")

        colors = itertools.cycle(colorSet)

        """
        For Single Array Graph one at a time but for which I also want to keep track of color, change individual draw by draw
        """
        self.colorCounter = self.colorCounter + 1
        for colorCntr in range(0, self.colorCounter, 1):
            colorUse = next(colors)

        """
        Special Types
        """
        if (graphType.lower() == "polygon"):
            self.graphingEachType(graphType, xData, yDataMat, keywords=keywords, color=colorUse, pylabUse=pylabUse)
        else:
            """
            Single Array Data Draw
            curDim = 0 if it is a single number
            """
            if (curDim <= 1):
                if (noLabel == True):
                    # pylabUse.plot(xData,yDataMat)
                    self.graphingEachType(graphType, xData, yDataMat, colorVar=colorVar, color=colorUse,
                                          keywords=keywords, pylabUse=pylabUse)
                if (noLabel == False):
                    # pylab.plot(xData,yDataMat,label=labelArray)
                    self.graphingEachType(graphType, xData, yDataMat, colorVar=colorVar, color=colorUse,
                                          keywords=keywords, label=labelArray, pylabUse=pylabUse)

            """
            Multiple Array Data Draw, with consistent color set
            """
            # colors = cm.rainbow(np.linspace(0, 1, len(ys)))
            if (curDim > 1):

                if (graphType.lower() == 'stackplot'):
                    # No Label here, Label in staplot legend specific section below
                    self.graphingEachType(graphType, xData, yDataMat, colorVar=colorVar,
                                          keywords=keywords, color=colorSet, pylabUse=pylabUse)
                else:
                    yColCount = yDataMat.shape[2 - 1]
                    colors = itertools.cycle(colorSet)
                    for curCol in range(0, yColCount):
                        color = next(colors)
                        if (noLabel == True):
                            # pylab.plot(xData,yDataMat[:,curCol])
                            self.graphingEachType(graphType, xData, yDataMat[:, curCol], colorVar=colorVar,
                                                  keywords=keywords, color=color, pylabUse=pylabUse)
                        if (noLabel == False):
                            # pylab.plot(xData,yDataMat[:,curCol],label=labelArray[curCol])
                            self.graphingEachType(graphType, xData, yDataMat[:, curCol], colorVar=colorVar,
                                                  keywords=keywords, label=labelArray[curCol], color=color,
                                                  pylabUse=pylabUse)

        """
        Labeling
        """
        if (noLabel == True):
            pass
        else:
            if (graphType.lower() == 'stackplot'):
                legendArtistList = []
                labelArrayUpdate = []
                rows, cols = yDataMat.shape
                for curStack in range(0, cols):
                    pCur = pylabUse.Rectangle((0, 0), 1, 1, fc=colorSet[curStack])
                    newLabel = labelArray[curStack] + \
                               ', ' + "{0:.{1}f}".format(yDataMat[0, curStack] * 100, 1) + '%' \
                                                                                           ' to ' + "{0:.{1}f}".format(
                        yDataMat[rows - 1, curStack] * 100, 1) + '%'
                    legendArtistList.append(pCur)
                    labelArrayUpdate.append(newLabel)
                pylabUse.legend(legendArtistList[::-1], labelArrayUpdate[::-1],
                                loc=labelLoc1t0, ncol=labelColCount, prop={'size': 6})
                pylabUse.axis((np.min(xData), np.max(xData), 0, 1.3))
            else:
                pylabUse.legend(loc=labelLoc1t0, ncol=labelColCount, prop={'size': 6})

        try:
            pylabUse.set_title(basicTitle)
            pylabUse.set_xlabel(basicXLabel)
            pylabUse.set_xlabel(basicYLabel)
        except:
            pylab.title(basicTitle)
            pylab.xlabel(basicXLabel)
            pylab.ylabel(basicYLabel)

        # =======================================================================
        # pylabUse.xticks()
        # ax.set_xticks(numpy.arange(0,1,0.1))
        # =======================================================================
        # pylab.grid()
        pylab.gca().grid(color='k', linestyle=':', linewidth=1, alpha=0.5)

        if (line45Deg == True):
            ymin = np.min(yDataMat)
            ymax = np.max(yDataMat)
            xmin = np.min(xData)
            xmax = np.max(xData)
            xymin = np.minimum(ymin, xmin)
            xymax = np.minimum(ymax, xmax)
            pylabUse.plot([xymin, xymax], [xymin, xymax], 'k--', alpha=0.5)

        if (ylim != None):
            pylabUse.ylim(ylim)

        if (xlim != None):
            pylabUse.xlim(xlim)

        # =======================================================================
        # try:
        #     pylab.ylim([np.min(yDataMat),np.max(yDataMat)])
        #     pylab.xlim([np.min(xData),np.max(xData)])
        # except:
        #     pass
        # =======================================================================

        if (showOrNot == True):
            pylab.show()

        if (saveOrNot == True):
            self.savingFig(saveDirectory, saveFileName, saveDPI, pylabUse=pylabUse, toScale=toScale)

        return pylabUse

    def graphingEachType(self, graphType, xSingleArrayData, ySingleArrayata,
                         keywords, colorVar=None, label=False, color='b', pylabUse=None):
        """
        If do not use basic pylab, but have external axis
        """
        if (pylabUse == None):
            pylabUse = pylab

        keys = sorted(keywords.keys())
        alpha = 1
        bins = 30
        scattersize = 1
        cmapColor = 'coolwarm'
        for kw in keys:
            if (kw == 'bins'):
                bins = int(keywords[kw])
            if (kw == 'scattersize'):
                scattersize = int(keywords[kw])
            if (kw == 'alpha'):
                alpha = float(keywords[kw])
            if (kw == 'cmap'):
                cmapColor = str(keywords[kw])

        if (label == False):
            label = ''

        if (graphType.lower() == 'polygon'):

            # ===================================================================
            # pylab.axes()
            # #points = [[2, 1], [8, 1], [8, 4]]
            # polygon = pylab.Polygon(ySingleArrayata)
            # #pylabUse.Polygon([[0,0],[4,1.1],[6,2.5],[2,1.4]], closed=True,color=color,fill=True,label=label)
            # pylab.gca().add_patch(polygon)
            # pylab.axis('scaled')
            # pylabUse.show()
            # ===================================================================
            # points = [[2, 1], [8, 1], [8, 4]]
            polygon = pylabUse.Polygon(ySingleArrayata, ec='k', fc=color, linewidth=1.5, alpha=0.3, label=label)
            # pylabUse.Polygon([[0,0],[4,1.1],[6,2.5],[2,1.4]], closed=True,color=color,fill=True,label=label)
            pylabUse.gca().add_patch(polygon)
            try:
                pylabUse.axis('scaled')
            except:
                pass

            # ===================================================================
            # try:
            #     pylabUse.add_patch(Polygon(ySingleArrayata,ec='k',fc=color, linewidth=1.5,alpha=0.3))
            # except:
            #     #points = [[2, 1], [8, 1], [8, 4]]
            #     polygon = pylabUse.Polygon(ySingleArrayata,ec='k',fc=color, linewidth=1.5,alpha=0.3)
            #     #pylabUse.Polygon([[0,0],[4,1.1],[6,2.5],[2,1.4]], closed=True,color=color,fill=True,label=label)
            #     pylabUse.gca().add_patch(polygon)
            #     #===============================================================
            #     # pylabUse.axis('scaled')
            #     #===============================================================
            # ===================================================================

            # ===================================================================
            # pylab.axis('scaled')
            # ===================================================================

            # ===================================================================
            # self.savingFig('C:/Users/fan/DynammicProgrammingPython/src/ProjectDisertCredit/model1/HandTry',
            #                'testPolygon', 100, pylabUse = pylabUse)
            # ===================================================================

        # ==============================================================================
        #            circle = pylab.Circle((0, 0), 0.75, fc='y')
        #            pylab.gca().add_patch(circle)
        #
        #            rectangle = pylab.Rectangle((10, 10), 100, 100, fc='r')
        #            pylab.gca().add_patch(rectangle)
        # ==============================================================================

        # ===============================================================================
        #             points = [[2, 1], [8, 1], [8, 4]]
        #             polygon = pylab.Polygon(points)
        #             pylab.gca().add_patch(polygon)
        #
        #             pylab.axis('scaled')
        #             pylab.show()
        #
        #
        #             points = [[2, 1], [8, 1], [8, 4]]
        #             pylab.Polygon(points)
        #             pylab.axis('scaled')
        #
        #             pylabUse.show()
        #
        #             #===================================================================
        #             # curPoly = Polygon([[0,0],[4,1.1],[6,2.5],[2,1.4]], closed=True,
        #             #           fill=False, hatch='/')
        #             # pylabUse.show()
        #             #===================================================================
        #
        #             # pylab.gca().add_patch(Polygon([[0,0],[4,1.1],[6,2.5],[2,1.4]], closed=True,fill=False, hatch='/'))
        # ===============================================================================

        # ===============================================================================
        #
        #             try:
        #                 points = [[2, 1], [8, 1], [8, 4]]
        #                 polygon = pylab.Polygon(points)
        #                 pylabUse.Polygon([[0,0],[4,1.1],[6,2.5],[2,1.4]], closed=True,color=color,fill=True,label=label)
        #             except:
        #                 pylabUse.add_patch(Polygon([[0,0],[4,1.1],[6,2.5],[2,1.4]], closed=True,color=color,fill=True,label=label))
        #             pylabUse.show()
        # ===============================================================================

        # pylab.gca().add_patch(Polygon(ySingleArrayata, closed=True,color=color,fill=True,label=label))
        if (graphType.lower() == 'stackplot'):
            pylabUse.stackplot(xSingleArrayData, np.transpose(ySingleArrayata), colors=color, alpha=alpha)
        if (graphType.lower() == 'density'):
            guassian_kde_graph(ySingleArrayata, color=color, label=label)
        if (graphType.lower() == 'plot'):
            pylabUse.plot(xSingleArrayData, ySingleArrayata, c=color, label=label, alpha=alpha)
        if (graphType.lower() == 'plotscatter'):
            pylabUse.plot(xSingleArrayData, ySingleArrayata, c=color, label=label)
            pylabUse.scatter(xSingleArrayData, ySingleArrayata, color='k', s=scattersize)
        if (graphType.lower() == 'scatter'):
            logger.debug('graphType.lower():%s', graphType.lower())

            if (colorVar is not None):
                # ===============================================================
                # pylabUse.scatter(xSingleArrayData, ySingleArrayata,marker='+', s=scattersize,
                #                  linewidths=4,c=colorVar)
                # pylabUse.gray()
                # ===============================================================

                # ===============================================================
                # coolwarm
                # ===============================================================
                # http://matplotlib.org/examples/color/colormaps_reference.html
                # ===============================================================
                # ===============================================================

                pylabUse.scatter(xSingleArrayData, ySingleArrayata, marker='+', s=scattersize,
                                 linewidths=4, c=colorVar,
                                 cmap=pylab.get_cmap(cmapColor), alpha=0.5)
            else:
                #                 logger.debug('xSingleArrayData:%s',xSingleArrayData)
                #                 logger.debug('ySingleArrayata:%s',ySingleArrayata)
                logger.debug('color:%s', color)
                logger.debug('label:%s', label)
                logger.debug('scattersize:%s', scattersize)
                pylabUse.scatter(xSingleArrayData, ySingleArrayata,
                                 color=color, label=label, s=scattersize, alpha=0.5)

        if (graphType.lower() == 'scatterregline'):
            pylabUse.scatter(xSingleArrayData, ySingleArrayata, s=scattersize)
            x = xSingleArrayData
            y = ySingleArrayata
            fit = pylabUse.polyfit(x, y, 1)
            fit_fn = pylabUse.poly1d(fit)  # fit_fn is now a function which takes in x and returns an estimate for y
            pylabUse.plot(x, y, 'yo', x, fit_fn(x), '--k', label=label)
        if (graphType.lower() == 'hist'):
            pylabUse.hist(ySingleArrayata, bins=bins, label=label, alpha=0.4)
        if (graphType.lower() == 'regline'):
            x = xSingleArrayData
            y = ySingleArrayata
            fit = pylabUse.polyfit(x, y, 1)
            fit_fn = pylabUse.poly1d(fit)  # fit_fn is now a function which takes in x and returns an estimate for y
            pylabUse.plot(x, y, 'yo', x, fit_fn(x), '--k', label=label)

        # =======================================================================
        # if (label == False):
        #     if(graphType == 'stackplot'):
        #         pylabUse.stackplot(xSingleArrayData, np.transpose(ySingleArrayata), colors=color, alpha=alpha)
        #     if(graphType == 'density'):
        #         guassian_kde_graph(ySingleArrayata, color=color)
        #     if(graphType == 'plot'):
        #         pylabUse.plot(xSingleArrayData, ySingleArrayata)
        #     if(graphType == 'plotscatter'):
        #         pylabUse.plot(xSingleArrayData, ySingleArrayata, c=color, label=label)
        #         pylabUse.scatter(xSingleArrayData, ySingleArrayata, c=color, s=scattersize)
        #     if(graphType == 'scatter'):
        #         pylabUse.scatter(xSingleArrayData, ySingleArrayata, c=color, s=scattersize)
        #     if(graphType == 'scatterregline'):
        #         pylabUse.scatter(xSingleArrayData, ySingleArrayata, s=2)
        #         x = xSingleArrayData
        #         y = ySingleArrayata
        #         fit = pylabUse.polyfit(x, y, 1)
        #         fit_fn = pylabUse.poly1d(fit)  # fit_fn is now a function which takes in x and returns an estimate for y
        #         pylabUse.plot(x, y, 'yo', x, fit_fn(x), '--k')
        #     if(graphType == 'hist'):
        #         pylabUse.hist(ySingleArrayata, bins=bins, alpha=0.4)
        #     if(graphType == 'regline'):
        #         x = xSingleArrayData
        #         y = ySingleArrayata
        #         fit = pylabUse.polyfit(x, y, 1)
        #         fit_fn = pylabUse.poly1d(fit)  # fit_fn is now a function which takes in x and returns an estimate for y
        #         pylabUse.plot(x, y, 'yo', x, fit_fn(x), '--k')
        # else :
        #     if(graphType == 'stackplot'):
        #         pylabUse.stackplot(xSingleArrayData, np.transpose(ySingleArrayata), colors=color, alpha=alpha)
        #     if(graphType == 'density'):
        #         guassian_kde_graph(ySingleArrayata, color=color, label=label)
        #     if(graphType == 'plot'):
        #         pylabUse.plot(xSingleArrayData, ySingleArrayata, c=color, label=label)
        #     if(graphType == 'plotscatter'):
        #         pylabUse.plot(xSingleArrayData, ySingleArrayata, c=color, label=label)
        #         pylabUse.scatter(xSingleArrayData, ySingleArrayata, c='k', s=scattersize)
        #     if(graphType == 'scatter'):
        #         pylabUse.scatter(xSingleArrayData, ySingleArrayata, c=color, label=label, s=scattersize)
        #     if(graphType == 'scatterregline'):
        #         pylabUse.scatter(xSingleArrayData, ySingleArrayata, s=scattersize)
        #         x = xSingleArrayData
        #         y = ySingleArrayata
        #         fit = pylabUse.polyfit(x, y, 1)
        #         fit_fn = pylabUse.poly1d(fit)  # fit_fn is now a function which takes in x and returns an estimate for y
        #         pylabUse.plot(x, y, 'yo', x, fit_fn(x), '--k', label=label)
        #     if(graphType == 'hist'):
        #         pylabUse.hist(ySingleArrayata, bins=bins, label=label, alpha=0.4)
        #     if(graphType == 'regline'):
        #         x = xSingleArrayData
        #         y = ySingleArrayata
        #         fit = pylabUse.polyfit(x, y, 1)
        #         fit_fn = pylabUse.poly1d(fit)  # fit_fn is now a function which takes in x and returns an estimate for y
        #         pylabUse.plot(x, y, 'yo', x, fit_fn(x), '--k', label=label)
        # =======================================================================

    def savingFig(self, saveDirectory=saveDirectory, saveFileName=saveFileName, saveDPI=saveDPI, saveOrNot=True,
                  showOrNot=False,
                  pylabUse=None, toScale=True, subplots_adjust=True):

        if (pylabUse == None):
            pylabUse = pylab

        if (subplots_adjust == True):
            try:
                pylabUse.subplots_adjust(hspace=0)
            except:
                pass

        if (pylabUse == None):
            pylabUse = pylab

        if (toScale == True):
            try:
                pylabUse.axis('scaled')
            except:
                pass

        try:
            pylabUse.tight_layout()
        except:
            pass

        font = {'family': 'sans-serif',
                'weight': 'normal',
                'size': 7}
        pylab.rc('font', **font)
        if (showOrNot == True):
            pylab.show()

        if (saveOrNot == True):
            pylab.savefig(saveDirectory + saveFileName, dpi=saveDPI, papertype='a4')

    def sampleGraphs(self, graphSampleType, graphType='plot'):

        if (graphSampleType == 'OnePlot'):
            self.xyPlotMultiYOneX(saveOrNot=False, graphType=graphType, labelArray=self.labelArray, noLabel=False)

        if (graphSampleType == 'TwoPlot'):
            pylab.subplot(2, 1, 1)
            self.xyPlotMultiYOneX(saveOrNot=False)
            pylab.subplot(2, 1, 2)
            self.xyPlotMultiYOneX(saveOrNot=False)

        if (graphSampleType == 'ScatterHistPlotDifferSize'):
            pylab.subplot2grid((3, 2), (0, 0), colspan=2, rowspan=2)
            self.xyPlotMultiYOneX(saveOrNot=False, graphType='scatter', scattersize=50)
            pylab.subplot2grid((3, 2), (2, 0))
            self.xyPlotMultiYOneX(saveOrNot=False, graphType='hist', bins=50)
            pylab.subplot2grid((3, 2), (2, 1))
            self.xyPlotMultiYOneX(saveOrNot=False, graphType='plot')

        if (graphSampleType == 'ScatterHistPlotDifferSize'):
            pylab.subplot2grid((3, 2), (0, 0), colspan=2, rowspan=2)
            self.xyPlotMultiYOneX(saveOrNot=False, graphType='scatter')
            pylab.subplot2grid((3, 2), (2, 0))
            self.xyPlotMultiYOneX(saveOrNot=False, graphType='hist', bins=20)
            pylab.subplot2grid((3, 2), (2, 1))
            self.xyPlotMultiYOneX(saveOrNot=False, graphType='plot')

        self.savingFig(saveFileName=graphSampleType, saveOrNot=False, showOrNot=True)
        pylab.clf()


def guassian_kde_graph(data_fordensity, graph_xgrid=False, xgridpoints=1000, color='b', label=False,
                       showOnScreen=False):
    density = gaussian_kde(data_fordensity)
    density.covariance_factor = lambda: .25
    density._compute_covariance()

    if (graph_xgrid == False):
        data_min = np.min(data_fordensity)
        data_max = np.max(data_fordensity)
        graph_xgrid = np.linspace(data_min, data_max, xgridpoints)

    if (label == False):
        pylab.plot(graph_xgrid, density(graph_xgrid), c=color)
    else:
        pylab.plot(graph_xgrid, density(graph_xgrid), c=color, label=label)

    if (showOnScreen == True):
        pylab.show()


def subplot_square_counter(totalimages=15):
    # ===========================================================================
    # for i in np.arange(9,16,1):  subplot_square_counter(totalimages=i)
    # ===========================================================================

    sqrt_image = np.sqrt(totalimages)
    sqrt_image = np.ceil(sqrt_image)

    sqrt_image_remainder = np.remainder(totalimages, sqrt_image)
    sqrt_image_divide = np.floor(totalimages / sqrt_image)

    cols = sqrt_image
    rows = sqrt_image_divide

    if (sqrt_image_remainder > 0):
        rows += 1

    return rows, cols


def sampleDataGraphs():
    # ===========================================================================
    # import Support.GraphSupport as grhSup
    # import pylab as pylab
    # grapher = grhSup.graphFunc()
    # ===========================================================================

    grapher = graphFunc()

    # x = np.arange(0, 3*np.pi+np.pi/4, 2*np.pi/8)
    pointCount = 500
    x = np.linspace(0, 3 * np.pi + np.pi / 4, pointCount)

    y_sin = np.sin(x)
    y_cos = np.cos(x)
    yColl = np.column_stack((y_sin, y_cos))

    np.random.seed(111)
    error = np.random.normal(loc=0, scale=1, size=len(x))
    y_sin_err = y_sin + error
    y_cos_err = y_cos + error

    """
    Simple Graph 1
    """
    basicTitle = 'Graph Test'
    basicXLabel = 'X'
    basicYLabel = 'Y'
    saveDirectory = 'C:\\Users\\fan\\Documents\\Dropbox (UH-ECON)\\Programming\\PYTHON\\TestGraphs\\'

    pylab.clf()
    saveFileName = 'graphTest1_line'
    grapher.xyPlotMultiYOneX(graphType='plot',
                             yDataMat=y_sin, xData=x,
                             basicTitle=basicTitle, basicXLabel=basicXLabel, basicYLabel=basicYLabel, noLabel=True,
                             saveOrNot=True, saveDirectory=saveDirectory, saveFileName=saveFileName)

    pylab.clf()
    saveFileName = 'graphTest1_scatter'
    grapher.xyPlotMultiYOneX(graphType='scatter', scattersize=10,
                             yDataMat=y_sin_err, xData=x,
                             basicTitle=basicTitle, basicXLabel=basicXLabel, basicYLabel=basicYLabel, noLabel=True,
                             saveOrNot=True, saveDirectory=saveDirectory, saveFileName=saveFileName)

    """
    Simple Graph 2
    """
    basicTitle = 'Graph Test'
    basicXLabel = 'X'
    basicYLabel = 'Y'
    saveDirectory = 'C:\\Users\\fan\\Documents\\Dropbox (UH-ECON)\\Programming\\PYTHON\\TestGraphs\\'

    pylab.clf()
    saveFileName = 'graphTest2_line'
    grapher.xyPlotMultiYOneX(graphType='plot',
                             yDataMat=yColl, xData=x,
                             labelArray=['sin', 'cos'], noLabel=False, labelLoc1t0=2, labelColCount=2,
                             basicTitle=basicTitle, basicXLabel=basicXLabel, basicYLabel=basicYLabel,
                             saveOrNot=True, saveDirectory=saveDirectory, saveFileName=saveFileName)

    pylab.clf()
    saveFileName = 'graphTest2_scatter'
    grapher.xyPlotMultiYOneX(graphType='scatter', scattersize=10,
                             yDataMat=yColl, xData=x,
                             labelArray=['sin', 'cos'], noLabel=False, labelLoc1t0=1, labelColCount=1,
                             basicTitle=basicTitle, basicXLabel=basicXLabel, basicYLabel=basicYLabel,
                             saveOrNot=True, saveDirectory=saveDirectory, saveFileName=saveFileName)

    """
    Simple Graph 3
    """
    graphTitleDisp = 'Graph Test 3D'
    xLabStr = 'X'
    yLabStr = 'Y'
    zLabStr = 'Z'

    zData = y_cos_err
    xData = y_cos
    yData = error

    saveDirectory = 'C:\\Users\\fan\\Documents\\Dropbox (UH-ECON)\\Programming\\PYTHON\\TestGraphs\\'
    graphTitleSave = saveDirectory + 'graphTest3_3d'
    contourAnd3D(xData, yData, zData,
                 xLabStr, yLabStr, zLabStr,
                 graphTitleDisp, graphTitleSave,
                 savedpi=125, angleType=[1, [1, 2, 3, 4, 5, 6]],
                 drawContour=True, draw3D=True,
                 contourXres=100, contourYres=100)

    # ===========================================================================
    # basicTitle = 'Graph Test'
    # basicXLabel = 'X'
    # basicYLabel = 'Y'
    # saveDirectory = 'C:\\Users\\fan\\Documents\\Dropbox\\Programming\\PYTHON\\TestGraphs\\'
    # saveFileName = 'graphTest1'
    # grapher.xyPlotMultiYOneX(graphType='plot',yDataMat=y1,xData=x,
    #                          basicTitle=basicTitle, basicXLabel=basicXLabel, basicYLabel=basicYLabel,noLabel=True,
    #                          saveOrNot=True, saveDirectory=saveDirectory,saveFileName=saveFileName)
    # ===========================================================================

    # ===========================================================================
    # grapher.savingFig(saveDirectory=self.support_args['IO'],saveFileName='graphMeanChoice_r'+str(self.support_args['cur_round']))
    # ===========================================================================


if __name__ == '__main__':
    # ===========================================================================
    # data = [1.5]*7 + [2.5]*2 + [3.5]*8 + [4.5]*3 + [5.5]*1 + [6.5]*8
    # density = gaussian_kde(data)
    # xs = np.linspace(0,8,200)
    # density.covariance_factor = lambda : .25
    # density._compute_covariance()
    # if (showOnScreen==True):
    #     pylab.plot(xs,density(xs))
    #     pylab.show()
    # ===========================================================================

    # ===============================================================================
    # guassian_kde_graph(showOnScreen=True)
    # ===============================================================================
    grapher = graphFunc()
    grapher.sampleGraphs(graphSampleType='OnePlot', graphType='scatter')
    grapher.sampleGraphs(graphSampleType = 'OnePlot',graphType='stackplot')
    grapher.sampleGraphs(graphSampleType = 'TwoPlot')
    grapher.sampleGraphs(graphSampleType = 'ScatterHistPlotDifferSize')

    # ===============================================================================
    # grapher = graphFunc()
    # grapher.sampleGraphs(graphSampleType = 'ScatterHistPlotDifferSize')
    # ===============================================================================



from pathlib import Path


def fp_search_rglob(spt_root="../",
                    st_rglob='*.py',
                    ls_srt_subfolders=None,
                    verbose=False,
                    ):
    """Searches for files with search string in a list of folders

    Parameters
    ----------
    spt_root: string
        root folder to search in
    st_rglob: string
        search for files with this rglob string
    ls_srt_subfolders: :obj:`list` of :obj:`str`
        a list of subfolders of the spt_root to search in
    verbose: bool
        print details

    Returns
    -------
    :obj:`list` of :obj:`str`
        A list of file names

    Examples
    --------

    >>> ls_spn = fp_search_rglob(spt_root="../",
    >>>                          ls_srt_subfolders=['rmd', 'pdf'],
    >>>                          st_rglob = '*d*.py')
    [WindowsPath('../rmd/bookdownparse.py'), WindowsPath('../rmd/mattexmd.py'), WindowsPath('../rmd/rmdparse.py'), WindowsPath('../pdf/pdfgen.py')]

    """

    # Directories to search in
    if ls_srt_subfolders is None:
        # ls_srt_subfolders = ['calconevar/', 'derivative/', 'derivative_application/',
        #                      'matrix_basics/',
        #                      'opti_firm_constrained/', 'opti_hh_constrained_brsv/',
        #                      'opti_hh_constrained_brsv_inequality/']
        # ls_srt_subfolders = ['matrix_basics/']
        ls_spt = [spt_root]
    else:
        ls_spt = [spt_root + srt_subf for srt_subf in ls_srt_subfolders]

    if verbose:
        print(ls_spt)

    # get file names
    ls_spn_found_tex = [spn_file for spt_srh in ls_spt
                        for spn_file in Path(spt_srh).rglob(st_rglob)]
    if verbose:
        print(ls_spn_found_tex)

    return ls_spn_found_tex


if __name__ == '__main__':
    spt_root_u = "../"
    ls_srt_subfolders_u = ['rmd', 'pdf']
    st_rglob_u = '*d*.py'
    verbose_u = False
    ls_spn = fp_search_rglob(spt_root=spt_root_u, ls_srt_subfolders=ls_srt_subfolders_u,
                             st_rglob=st_rglob_u, verbose=verbose_u)
    print(ls_spn)




"""Generate README from Fan Rmd Frontmatter

Generates README.md TOC contents based on parsed YAML frontmatter from files listed
in a bookdown yaml file that contains a list of RMD files to process through.
"""

import yaml
import frontmatter
import pyfan.util.rmd.rmdparse as rmdparse
import os


def fs_yml2readme(sfc_prj='R4Econ',
                  sph_prj='C:/Users/fan/R4Econ/',
                  spn_prj_bookdown_yml='_bookdown.yml',
                  spn_prj_readme_toc='README_toc.md',
                  ls_st_ignore=['index.Rmd', 'README_appendix.md', 'title.Rmd', 'main.Rmd'],
                  sph_pdf='htmlpdfr',
                  sph_html='htmlpdfr',
                  sph_r='htmlpdfr',
                  st_file_type='r',
                  verbose=False):
    """Write to file README detailed TOC for files in bookdown yaml list

    Parameters
    ----------
    sfc_prj : string
        The git project name
    sph_prj : string
        the local path to the git project
    spn_prj_bookdown_yml : string
        yml file name under project root contains rmd names under 'rmd_files' key
    spn_prj_readme_toc : string
        md generated file name under project root
    ls_st_ignore: list
        list of string names to ignore
    sph_pdf : string
        subfolder to store pdf files in the rmd folder
    sph_html : string
        subfolder to store html files in the rmd folder
    sph_r : string
        subfolder to store r files in the rmd folder does not
        have to be r, any other raw file type, m of py for example
    st_file_type: string
        the RMD file is for which underlying language: r for R, m for matlab, py
        for python
    verbose: bool
        print details

    Returns
    -------
    None
        nothing is returned, the spn_prj_readme_toc toc file is generated

    Examples
    --------

    >>> fs_yml2readme(sfc_prj='pyfan', sph_prj='../../../', verbose=False)

    """

    spn_readme = sph_prj + spn_prj_readme_toc
    f_core = open(spn_readme, "w")

    spn_yml = sph_prj + spn_prj_bookdown_yml
    with open(spn_yml, 'r') as f:
        ob_book_yml = yaml.load(f, Loader=yaml.BaseLoader)

    ls_st_rmd_files = ob_book_yml['rmd_files']

    sfc_main = 'main.Rmd'
    ctr_chapter = 0
    ctr_section = 0
    ctr_subsection = 0

    for sfc in ls_st_rmd_files:

        if sfc in ls_st_ignore:
            if verbose:
                print('ignore:' + sfc)

        elif sfc_main in sfc:

            # Load _main File
            spn_rmd_main = sph_prj + sfc
            ob_yml_rmd_main = frontmatter.load(spn_rmd_main)
            ob_yml_rmd_main = ob_yml_rmd_main.content + '\n\n'

            # section heading with proper numbering
            it_pound = ob_yml_rmd_main.count("#")
            if it_pound == 1:
                ctr_chapter = ctr_chapter + 1
                ctr_section = 0
                ctr_subsection = 0
                st_text_out = '# ' + str(ctr_chapter) + ' ' + \
                              ob_yml_rmd_main.replace("#", "")
                if ctr_chapter != 1:
                    st_text_out = '\n' + st_text_out
            elif it_pound == 2:
                ctr_section = ctr_section + 1
                ctr_subsection = 0
                st_text_out = '## ' + str(ctr_chapter) + '.' + str(ctr_section) + \
                              ' ' + ob_yml_rmd_main.replace("#", "")
                if ctr_section != 1:
                    st_text_out = '\n' + st_text_out
            # write to file
            if verbose:
                print(st_text_out)
            f_core.write(st_text_out)

        else:

            # Load RMD front matter and summarize
            ctr_subsection = ctr_subsection + 1
            st_head_link, ls_desc_out, ls_code_out = \
                rmdparse.fs_rmd_yml_parse(
                    sfc_prj=sfc_prj, sph_prj=sph_prj, spn_prj_rmd=sfc,
                    sph_pdf=sph_pdf,
                    sph_html=sph_html,
                    sph_r=sph_r,
                    st_file_type=st_file_type)

            # write to file file title and main
            st_text_out = str(ctr_subsection) + '. ' + st_head_link + '\n'
            if verbose:
                print(st_text_out)
            f_core.write(st_text_out)

            # write to file file desc
            for st_desc in ls_desc_out:
                f_core.write(st_desc + '\n')
            for st_code in ls_code_out:
                f_core.write(st_code + '\n')

    # ls_front_keys = ob_book_yml.keys()
    # print(ls_front_keys)

    f_core.close()


if __name__ == '__main__':
    # sfc_prj = 'R4Econ'
    # sph_prj = 'C:/Users/fan/R4Econ/'
    # spn_prj_bookdown_yml = '_bookdown.yml'
    # spn_prj_readme_toc = 'README_toc_two.md'
    #
    # fs_yml2readme(sfc_prj=sfc_prj,
    #               sph_prj=sph_prj,
    #               spn_prj_bookdown_yml=spn_prj_bookdown_yml,
    #               spn_prj_readme_toc=spn_prj_readme_toc)

    # fs_yml2readme(sfc_prj='pyfan', sph_prj='../../../', verbose=False)

    fs_yml2readme(sfc_prj='Math4Econ',
                  sph_prj='C:/Users/fan/Math4Econ/',
                  spn_prj_bookdown_yml='_bookdown.yml',
                  spn_prj_readme_toc='README_toc.md',
                  ls_st_ignore=['index.Rmd', 'README_appendix.md', 'title.Rmd', 'main.Rmd'],
                  sph_pdf='htmlpdfm',
                  sph_html='htmlpdfm',
                  sph_r='htmlpdfm',
                  st_file_type='m',
                  verbose=True)



'''
Created on Mar 16, 2017

@author: fan
'''

def dynamic_obj_attr(attribute_array=['r_save', 'r_borr', 'delta'],
                     attribute_values_array=['0.02', '0.05', '0.10'],
                     print_values=False):

    class Object(object):
        pass
    standin_obj = Object()

    for index, param in enumerate(attribute_array):
        setattr(standin_obj, param, attribute_values_array[index])

    if(print_values == True):
        print(standin_obj.__dict__)

    return standin_obj

if __name__ == '__main__':

    dynamic_obj_attr(print_values=True)



'''
Created on Aug 6, 2018

@author: fan

Design page subplot

'''


def subplot_design(plot_count=10, base_multiple=4, base_multiple_high_frac = 0.60):
    """subplot grid and size given total plot count
    
    figsize = (width height)
    
    Examples
    --------
    import Support.graph.subplot as sup_graph_subplot
    figsize, rows, cols = sup_graph_subplot.subplot_design(plot_count=10, base_multiple=4, base_multiple_high_frac = 0.60)        
    """
    
    if plot_count == 1:
        figsize = (base_multiple*1,base_multiple*1)
        rows = 1
        cols = 1
    elif plot_count <= 2:    
        figsize = (base_multiple*2,base_multiple*1)
        rows = 1
        cols = 2
    elif plot_count <= 4:
        figsize = (base_multiple*2,base_multiple*2)
        rows = 2
        cols = 2
    elif plot_count <= 6:
        figsize = (base_multiple*3,base_multiple*2)
        rows = 2
        cols = 3
    elif plot_count <= 9:
        figsize = (base_multiple*3,base_multiple*3)
        rows = 3
        cols = 3        
    elif plot_count <= 12:
        figsize = (base_multiple*4,base_multiple*3)
        rows = 3
        cols = 4        
    elif plot_count <= 16:
        figsize = (base_multiple*4,base_multiple*4)
        rows = 4
        cols = 4
    elif plot_count <= 20:
        figsize = (base_multiple*5,base_multiple*4)
        rows = 4
        cols = 5
    elif plot_count <= 25:
        figsize = (base_multiple*5,base_multiple*5)
        rows = 5
        cols = 5
    else:
                
        lessthan = True        
        sqr_ctr = 1
        while lessthan:
            sqr_ctr += 1
            if (sqr_ctr**2 >= plot_count):
                lessthan = False
                minus_one_row = False
                if (sqr_ctr*(sqr_ctr-1) >= plot_count):
                    minus_one_row = True
                    
        rows = sqr_ctr
        if (minus_one_row):
            rows = rows - 1
        cols = sqr_ctr
    
        figsize_width = max(20, int(base_multiple*base_multiple_high_frac*rows))
        figsize_width = max(20, int(base_multiple*base_multiple_high_frac*cols))
        figsize = (figsize_width, figsize_width)
    
    return figsize, rows, cols

if __name__ == "__main__":
    for plot_count in range(100):
        print('plot_count:', plot_count, ', result:', subplot_design(plot_count=plot_count))
    



'''
Created on Aug 9, 2018

@author: fan

Store key variable names, file names, etc. 

Individual analysis files should refer to this file. 
'''
import logging
import pyfan.panda.categorical.catevars as tool_catevars

logger = logging.getLogger(__name__)


def main_data_directory(dataset_name='thai_data_z'):
    if (dataset_name == 'thai_data_z'):
        # this means I am running my home windows machine
        data_directory = 'C:/Users/fan/Documents/Dropbox (UH-ECON)/Project_Nguyen_Townsend_Wang_ThaiFinChoice/data_month160/'

    return data_directory


def file_names():
    data_z = {'filename': 'thaiMthly_Annulized_DataZ',
              'format': 'stata',
              'format_suffix': '.dta'}
    file_names_dict = {'thai_data_z': data_z}

    return file_names_dict


def var_names(vartype=1,
              file_name='thaiMthly_Annulized_DataZ',
              out_type='coln'):
    if (file_name == "thaiMthly_Annulized_DataZ"):
        vars_d = {'Y': ['hy_agg_IS_07', 'Income'],
                  'K': ['hy_agg_BS_09_1st', 'Capital'],
                  'B': ['B_i_y', 'Net Asset'],
                  'provid': ['id_p', 'province'],
                  'provid_num': ['id_p_num', 'province'],
                  'year': ['ts_yrr', 'calendar year']}

    if (out_type == 'coln'):
        vars_coln = {var_key: vars_val[0] for var_key, vars_val in vars_d.items()}
        return vars_coln

    if (out_type == 'desc'):
        vars_desc = {var_key: vars_val[1] for var_key, vars_val in vars_d.items()}
        return vars_desc


def conditions(df, condi_dict=None, return_subset=False):
    """Commonly used conditionings
    
    Can combine conditioning statements together in list
    
    Parameters
    ----------
    condi_dict: dictionary
        condi_dict = {'region':'central', 'years':2002-2005}
    file_name: string
        different files could have different variable names for the same variables,
        although these should be unified if possible
    """

    vars_d = df.vars_c

    '''
    1. Define Conditioning Functions
    '''

    def condi_region(region):
        #     {
        #         "0": "Chachoengsao (Central)",
        #         "1": "Buriram (NE)",
        #         "2": "Lopburi (Central)",
        #         "3": "Sisaket (NE)"
        #     }
        cates_dict = tool_catevars.show_cates(df, vars_d['provid_num'])
        if (region.lower() == 'central' or region.lower() == 'ne'):
            if_condi = ((df[vars_d['provid_num']] == cates_dict[0]) | (df[vars_d['provid_num']] == cates_dict[2]))
        if (region.lower() == 'northeast' or region.lower() == 'ne'):
            if_condi = ((df[vars_d['provid_num']] == cates_dict[1]) | (df[vars_d['provid_num']] == cates_dict[3]))
        return if_condi

    def condi_year(year_range):
        if '-' in year_range:
            year_min_max = year_range.split('-')
            year_min = int(year_min_max[0])
            year_max = int(year_min_max[1])
        if_condi = ((df[vars_d['year']] >= year_min) & (df[vars_d['year']] <= year_max))
        return if_condi

    for ctr, (condi_key, condi_val) in enumerate(condi_dict.items()):
        if (condi_key == 'region'):
            if_condi = condi_region(condi_val)
        if (condi_key == 'year'):
            if_condi = condi_year(condi_val)

        if (ctr == 0):
            if_condi_all = if_condi
        else:
            if_condi_all = if_condi_all & if_condi

    if (return_subset):
        '''
        return subset of data with conditioning imposed
        '''
        return df[if_condi_all]
    else:
        '''
        return just the conditioning
        '''
        return if_condi_all



import cdsapi
import urllib.request

# download folder
spt_root = "C:/Users/fan/Downloads/_data/"
spn_dl_test_grib = spt_root + "derived_utci_2010_2.zip"
# request
c = cdsapi.Client()

res = c.retrieve(
    'derived-utci-historical',
    {
        'format': 'zip',
        'variable': 'Universal thermal climate index',
        'product_type': 'Consolidated dataset',
        'year': '2010',
        'month': [
            '07',
        ],
        'day': [
            '01', '02'
        ],
        'area': [53.31, 73, 4.15, 135],
        'grid': [0.25, 0.25],
    },
    spn_dl_test_grib)

# show results
print('print results')
print(res)
print(type(res))



'''
Created on Nov 27, 2017

@author: fan
'''

import numpy as np
import scipy.stats

import pyfan.amto.array.mesh as mesh


def three_vec_grids(vara_min, vara_max, vara_grid, vara_grid_add=None,
                    varb_min=None, varb_max=None, varb_grid=None, varb_grid_add=None,
                    varc_min=None, varc_max=None, varc_grid=None, varc_grid_add=None,
                    gridtype='grid', tomesh=False,
                    return_joint=False, return_single_col=False,
                    seed=999):
    """Grid for VFI
    Temporary code, so that I can deal with minimal school hour. should be 
    deleted in the future. and combined with the method above 
    """

    if (gridtype == 'grid'):
        a = np.linspace(vara_min, vara_max, vara_grid)
        if (varb_grid is not None):
            b = np.linspace(varb_min, varb_max, varb_grid)
        if (varc_grid is not None):
            c = np.linspace(varc_min, varc_max, varc_grid)
            if (varc_grid_add is not None):
                c = np.append(c, varc_grid_add)

    if (gridtype == 'rand'):
        np.random.seed(seed)
        a = random_vector_min_max(vara_min, vara_max, vara_grid)
        if (varb_grid is not None):
            b = random_vector_min_max(varb_min, varb_max, varb_grid)
        if (varc_grid is not None):
            c = random_vector_min_max(varc_min, varc_max, varc_grid)

    if (vara_grid_add is not None):
        a = np.append(a, vara_grid_add)
    if (varb_grid_add is not None):
        b = np.append(b, varb_grid_add)
    if (varc_grid_add is not None):
        c = np.append(c, varc_grid_add)

    a = np.sort(a)
    a = np.reshape(a, (-1, 1))

    if (varb_grid is not None):
        b = np.sort(b)
        if (varb_grid is not None):
            b = np.reshape(b, (-1, 1))

    if (varc_grid is not None):
        c = np.sort(c)
        if (varc_grid is not None):
            c = np.reshape(c, (-1, 1))

    if (varb_grid is not None):
        if (varc_grid is not None):
            if (tomesh == True):
                if (return_joint is True):
                    abc = mesh.three_mat_mesh(a, b, c,
                                              return_joint=return_joint,
                                              return_single_col=return_single_col)
                    return abc
                else:
                    a, b, c = mesh.three_mat_mesh(a, b, c,
                                                  return_joint=return_joint,
                                                  return_single_col=return_single_col)
                    return a, b, c
            else:
                return a, b, c
        else:
            if (tomesh == True):
                if (return_joint is True):
                    ab = mesh.two_mat_mesh(a, b, return_joint,
                                           return_single_col=return_single_col)
                    return ab
                else:
                    a, b = mesh.two_mat_mesh(a, b, return_joint,
                                             return_single_col=return_single_col)
                    return a, b
            else:
                return a, b
    else:
        return a


def random_vector_mean_sd(mean, sd, grid_count, gridtype='grid', seed=382):
    if (gridtype == 'grid'):
        unif_grid = np.linspace(0.001, 0.999, grid_count)
        standard_normal_grid = scipy.stats.norm.ppf(unif_grid)
        standard_normal_grid = np.reshape(standard_normal_grid, (-1, 1))

    if (gridtype == 'rand'):
        np.random.seed(seed)
        standard_normal_grid = np.random.randn(grid_count, 1)

    normal_grid = mean + sd * standard_normal_grid

    return normal_grid


def random_vector_min_max(minval, maxval, grid_count):
    if (grid_count == 1):
        'grid_count == 1 means we have no choice along that dimension'
        random_points = np.random.uniform(grid_count)
        random_vector = minval + random_points * (maxval - minval)
    else:
        random_points = np.random.uniform(size=(grid_count - 2))
        random_vector = minval + random_points * (maxval - minval)

        random_vector = np.insert(random_vector, 0, minval)
        random_vector = np.insert(random_vector, len(random_vector), maxval)

    return random_vector


if __name__ == '__main__':
    vara_min = 1
    vara_max = 10
    vara_grid = 4

    varb_min = 30
    varb_max = 31
    varb_grid = 2

    varc_min = -10
    varc_max = -5
    varc_grid = 3

    print('3 grids, 3 outputs')
    [veca, vecb, vecc] = three_vec_grids(vara_min, vara_max, vara_grid,
                                         vara_grid_add=None,
                                         varb_min=varb_min, varb_max=varb_max, varb_grid=varb_grid,
                                         varb_grid_add=None,
                                         varc_min=varc_min, varc_max=varc_max, varc_grid=varc_grid,
                                         varc_grid_add=None,
                                         gridtype='grid', tomesh=False, return_joint=False, seed=999)
    print('vec a:', np.transpose(veca))
    print('vec b:', np.transpose(vecb))
    print('vec c:', np.transpose(vecc))

    print('3 grids, mesh, 3 outputs')
    [veca, vecb, vecc] = three_vec_grids(vara_min, vara_max, vara_grid,
                                         vara_grid_add=None,
                                         varb_min=varb_min, varb_max=varb_max, varb_grid=varb_grid,
                                         varb_grid_add=None,
                                         varc_min=varc_min, varc_max=varc_max, varc_grid=varc_grid,
                                         varc_grid_add=None,
                                         gridtype='grid', tomesh=True, return_joint=False, seed=999)
    print('vec a:', np.transpose(veca))
    print('vec b:', np.transpose(vecb))
    print('vec c:', np.transpose(vecc))

    print('3 grids, mesh, joint')
    outputs = three_vec_grids(vara_min, vara_max, vara_grid,
                              vara_grid_add=None,
                              varb_min=varb_min, varb_max=varb_max, varb_grid=varb_grid,
                              varb_grid_add=None,
                              varc_min=varc_min, varc_max=varc_max, varc_grid=varc_grid,
                              varc_grid_add=None,
                              gridtype='grid', tomesh=True, return_joint=True, seed=999)
    print('outputs:', outputs)

    print('3 grids, mesh, joint, rand')
    outputs = three_vec_grids(vara_min, vara_max, vara_grid,
                              vara_grid_add=None,
                              varb_min=varb_min, varb_max=varb_max, varb_grid=varb_grid,
                              varb_grid_add=None,
                              varc_min=varc_min, varc_max=varc_max, varc_grid=varc_grid,
                              varc_grid_add=None,
                              gridtype='rand', tomesh=True, return_joint=True, seed=999)
    print('outputs:', outputs)

    print('3 grids, mesh, joint, rand, add_more')
    outputs = three_vec_grids(vara_min, vara_max, vara_grid,
                              vara_grid_add=np.array([-1]),
                              varb_min=varb_min, varb_max=varb_max, varb_grid=varb_grid,
                              varb_grid_add=None,
                              varc_min=varc_min, varc_max=varc_max, varc_grid=varc_grid,
                              varc_grid_add=np.array([-999, -99, 111]),
                              gridtype='rand', tomesh=True, return_joint=True, seed=999)
    print('outputs:', outputs)

    print('2 grids, mesh, joint, rand, add_more')
    outputs = three_vec_grids(vara_min, vara_max, vara_grid,
                              vara_grid_add=np.array([-1, 839, 33]),
                              varb_min=varb_min, varb_max=varb_max, varb_grid=varb_grid,
                              varb_grid_add=None,
                              gridtype='grid', tomesh=True, return_joint=True, seed=999)
    print('outputs:', outputs)



"""Fan Rmd Fronter Matter Core Parser

In mine Rmd files front matter, there are some special parameters that
provide summaries for the key contents in the code, and also provide
key programs and dependencies that were used. The information can be parsed
to string lists. The special strings are meant to be used with a special
README.md github pages file. The goal is to automatically gather meta-data
across RMD files within folders in a package to prepare data needed to provide
a detailed table of content type contents needed for a README.md file.
"""

import frontmatter
import os


# sfc_prj='R4Econ'
# sph_prj='C:/Users/fan/R4Econ'
# spn_prj_rmd='/summarize/aggregate/fs_group_unique_agg.Rmd'
# sph_gitpages_root='https://fanwangecon.github.io/'
# sph_github_root='https://github.com/FanWangEcon/'
# sph_branch='/master'
# sph_pdf='/htmlpdfr'
# sph_html='/htmlpdfr'
# sph_r='/htmlpdfr'

def fs_rmd_yml_parse(sfc_prj='R4Econ',
                     sph_prj='C:/Users/fan/R4Econ',
                     spn_prj_rmd='/summarize/aggregate/fs_group_unique_agg.Rmd',
                     sph_gitpages_root='https://fanwangecon.github.io/',
                     sph_github_root='https://github.com/FanWangEcon/',
                     sph_branch='/master',
                     sph_pdf='htmlpdfr',
                     sph_html='htmlpdfr',
                     sph_r='htmlpdfr',
                     st_file_type='r'):
    """Parse Yaml Frontmatter to get list of paths, summaries, dependencies

    Parameters
    ----------
    sfc_prj : string
        The git project name
    sph_prj : string
        the local path to the git project
    spn_prj_rmd : string
        the path (within project) to the Rmd file
    sph_gitpages_root : string
        github pages site url root
    sph_github_root : string
        github project page url
    sph_branch : string
        which branch
    sph_pdf : string
        subfolder to store pdf files in the rmd folder
    sph_html : string
        subfolder to store html files in the rmd folder
    sph_r : string
        subfolder to store r files in the rmd folder does not
        have to be r, any other raw file type, m of py for example
    st_file_type: string
        the RMD file is for which underlying language: r for R, m for matlab, py
        for python
    """

    # 1. [Count Unique Groups and Mean within Groups](
    # https://fanwangecon.github.io/R4Econ/summarize/aggregate/fs_group_unique_agg.html): r \| ref \| [**rmd**](
    # https://github.com/FanWangEcon/R4Econ/blob/master/summarize/aggregate/fs_group_unique_agg.Rmd ) \| [**pdf**](
    # https://github.com/FanWangEcon/R4Econ/blob/master/summarize/aggregate/fs_group_unique_agg.pdf) \| [**html**](
    # https://fanwangecon.github.io/R4Econ/summarize/aggregate/fs_group_unique_agg.html)

    spn_rmd = sph_prj + spn_prj_rmd
    ob_yml_rmd = frontmatter.load(spn_rmd)
    ls_front_keys = ob_yml_rmd.keys()

    # Construct RMD R PDF HTML Path
    sfc_rmd_file = os.path.basename(spn_rmd)
    sfc_file_base = os.path.splitext(sfc_rmd_file)[0]
    spt_rmd_path = os.path.dirname(spn_prj_rmd)
    if 'titleshort' in ls_front_keys:
        st_title = ob_yml_rmd['titleshort']
    elif 'title' in ls_front_keys:
        st_title = ob_yml_rmd['title']
    else:
        st_title = sfc_file_base

    sph_source_blob_root = sph_github_root + sfc_prj + '/blob' + sph_branch + '/' + spt_rmd_path + '/'
    sph_rmd_pdf = sph_source_blob_root + sph_pdf + '/' + sfc_file_base + '.pdf'
    sph_rmd_rmd = sph_source_blob_root + '/' + sfc_file_base + '.Rmd'

    sph_source_web_root = sph_gitpages_root + sfc_prj + '/' + spt_rmd_path + '/'
    sph_rmd_html = sph_source_web_root + sph_html + '/' + sfc_file_base + '.html'

    st_head_link = '[' + st_title + '](' + sph_rmd_html + '):'

    if st_file_type == 'r':

        sph_rmd_r = sph_source_blob_root + sph_r + '/' + sfc_file_base + '.R'

        st_head_link = st_head_link + ' [**rmd**](' + sph_rmd_rmd + ')'
        st_head_link = st_head_link + ' \\| [**r**](' + sph_rmd_r + ')'
        st_head_link = st_head_link + ' \\| [**pdf**](' + sph_rmd_pdf + ')'
        st_head_link = st_head_link + ' \\| [**html**](' + sph_rmd_html + ')'
    elif st_file_type == 'm':

        sph_rmd_m = sph_source_blob_root + sph_r + '/' + sfc_file_base + '.m'
        # assume mlx and rmd same folder
        sph_rmd_mlx = sph_source_blob_root + sfc_file_base + '.mlx'

        st_head_link = st_head_link + ' [**mlx**](' + sph_rmd_mlx + ')'
        st_head_link = st_head_link + ' \\| [**m**](' + sph_rmd_m + ')'
        st_head_link = st_head_link + ' \\| [**pdf**](' + sph_rmd_pdf + ')'
        st_head_link = st_head_link + ' \\| [**html**](' + sph_rmd_html + ')'
    elif st_file_type == 'py':
        st_head_link = st_head_link + ' [**rmd**](' + sph_rmd_rmd + ')'
        # st_head_link = st_head_link + ' \\| [**m**](' + sph_rmd_r + ')'
        st_head_link = st_head_link + ' \\| [**pdf**](' + sph_rmd_pdf + ')'
        st_head_link = st_head_link + ' \\| [**html**](' + sph_rmd_html + ')'

    # Function Details Bullet Points
    if 'description' in ls_front_keys:
        st_desc = ob_yml_rmd['description']
        ls_st_desc = st_desc.split('\n', 4)[:-1]
        ls_desc_out = [None] * len(ls_st_desc)
        for ctr in range(0, len(ls_st_desc)):
            st_ful = '\t+ ' + ls_st_desc[ctr]
            ls_desc_out[ctr] = st_ful
    else:
        ls_desc_out = []

    # Construct R Functions used string
    if 'core' in ls_front_keys:
        ls_summary = ob_yml_rmd['core']
        ls_code_out = [None] * len(ls_summary)
        for ctr in range(0, len(ls_summary)):
            st_str = '**' + ls_summary[ctr]['package'] + '**'
            st_end = '*' + " + ".join(ls_summary[ctr]['code'].split('\n')[:-1]) + '*'
            st_ful = '\t+ ' + st_str + ': ' + st_end
            ls_code_out[ctr] = st_ful
    else:
        ls_code_out = []

    # print(st_head_link)
    # print(ls_desc_out)
    # print(ls_code_out)

    return st_head_link, ls_desc_out, ls_code_out


if __name__ == '__main__':
    spn_prj_rmd = '/summarize/aggregate/fs_group_summ_wide.Rmd'
    spn_prj_rmd = '/summarize/aggregate/fs_group_unique_agg.Rmd'
    st_head_link_a, ls_desc_out_a, ls_code_out_a = fs_rmd_yml_parse(spn_prj_rmd=spn_prj_rmd)
    print(st_head_link_a)
    print(ls_desc_out_a)
    print(ls_code_out_a)



from pathlib import Path
import subprocess
import os
import yaml
import pyfan.util.path.getfiles as getfiles
import time


def fp_md2rmd(spt_root="C:/Users/fan/Math4Econ/",
              ls_srt_subfolders=['calconevar/'],
              snm_folder_yml='preamble.yml',
              st_yml_file_key='file',
              st_rglob_md='*.md', verbose=False):
    """Generate RMD from MD with YAML Header

    In each folder, there is a prior preamble.yaml for all files in folder.
    There are md files generated by the fp_mlxtex2md function, this file stacks the yaml
    header on top of the md after searching for the yml header with the no suffix
    file name of the md file, which is the search key in the yaml.

    Parameters
    ----------
    spt_root : string
        The root folder
    ls_srt_subfolders : :obj:`list` of :obj:`str`
        List of subfolders to search in
    snm_folder_yml: string
        name of the file containing all rmd yaml header under spt_root
    st_yml_file_key: string
        the key in the yaml header snm_folder_yml with file names no suffix
    st_rglob_md : string
        md files search string rglob
    verbose : bool
        print extra

    Returns
    -------
    None
        nothing is returned, mlx generated tex files updates, and pandoc md generated

    """

    ls_spn_found_md = \
        getfiles.fp_search_rglob(spt_root=spt_root, ls_srt_subfolders=ls_srt_subfolders,
                                 st_rglob=st_rglob_md, verbose=verbose)

    for spn_md in ls_spn_found_md:
        if verbose:
            print(spn_md)

        # spn_md = "C:/Users/fan/Math4Econ/matrix_basics/matrix_matlab.md"
        sna_file = Path(spn_md).stem
        spn_rmd = os.path.splitext(spn_md)[0] + '.Rmd'
        spn_yml = os.path.split(spn_md)[0] + '/' + snm_folder_yml

        # Open existing md file
        fl_md = open(spn_md)
        tx_fl_md = fl_md.read()

        # Link String
        st_link_line_1 = "**back to** [**Fan**](https://fanwangecon.github.io)**'s** [**Intro Math\n"
        st_link_line_2 = "for Econ**](https://fanwangecon.github.io/Math4Econ/)**,** [**Matlab\n"
        st_link_line_3 = "Examples**](https://fanwangecon.github.io/M4Econ/)**, or** [**Dynamic\n"
        st_link_line_4 = "Asset**](https://fanwangecon.github.io/CodeDynaAsset/) **Repositories**\n"
        st_link_lines = st_link_line_1 + st_link_line_2 + st_link_line_3 + st_link_line_4

        st_link_rep_1 = "```{r global_options, include = FALSE}\ntry(source('../.Rprofile'))\n```\n"
        st_link_rep_2 = "\n`r text_shared_preamble_one`\n`r text_shared_preamble_two`\n`r text_shared_preamble_thr`\n"

        # replace md elements
        ls_st_old = ['![image]', '.png)', st_link_line_1, st_link_line_2, st_link_line_3, st_link_line_4]
        ls_st_new = ['![]', '.png){width=500px}', st_link_rep_1, st_link_rep_2, '', '']

        # zip and loop and replace
        for old, new in zip(ls_st_old, ls_st_new):
            tx_fl_md = tx_fl_md.replace(old, new)
        if verbose:
            print(tx_fl_md)

        # Open and get yml component
        # Assume that there is a yml file under the immediate path
        fl_yml = open(spn_yml)
        ls_dict_yml = yaml.load(fl_yml, Loader=yaml.BaseLoader)
        if verbose:
            print(ls_dict_yml)

        # From the file with all folder file yamls, find the file specific yaml
        ls_dict_selected = [dict_yml for dict_yml in ls_dict_yml
                            if dict_yml[st_yml_file_key] in [sna_file]]

        # Put Overall String Structure Together
        st_rmd_yml = yaml.dump(ls_dict_selected[0])

        # Open new RMD file
        fl_rmd = open(spn_rmd, 'w')
        fl_rmd.write('---\n')
        fl_rmd.write(st_rmd_yml)
        fl_rmd.write('---\n\n')
        fl_rmd.write(tx_fl_md)

        # closing
        fl_md.close()
        fl_yml.close()
        fl_rmd.close()
        # print


def fp_mlxtex2md(spt_root="C:/Users/fan/Math4Econ/",
                 ls_srt_subfolders=['calconevar/'],
                 st_rglob_tex='*.tex', verbose=False):
    """Edit and replace MLX based tex and convert to markdown

    Several mlx auto converted tex elements need to be constructed, otherwise
    the md file would not work. Importantly, convert how images are stored to
    aggregate subfolder. Which relies on movefiles.py to generate to put
    in the image folder structure as specified here which is under an
    aggregate img folder.

    Parameters
    ----------
    spt_root : string
        The root folder
    ls_srt_subfolders : :obj:`list` of :obj:`str'
        List of subfolders to search in
    st_rglob_tex : string
        tex files search string rglob
    verbose : bool
        print extra

    Returns
    -------
    None
        nothing is returned, mlx generated tex files updates, and pandoc md generated

    Examples
    --------
    >>> fp_mlxtex2md(spt_root='C:/Users/fan/Math4Econ/matrix_application/',
    ...              ls_srt_subfolders=None, st_rglob_tex='twogoods.tex', verbose=True)

    """

    # spt_root = "C:/Users/fan/Math4Econ/"
    # ls_srt_subfolders = ['calconevar/', 'derivative/', 'derivative_application/',
    #                      'matrix_basics/',
    #                      'opti_firm_constrained/', 'opti_hh_constrained_brsv/',
    #                      'opti_hh_constrained_brsv_inequality/']
    # ls_srt_subfolders = ['matrix_basics/']
    # st_rglob_tex = '*.tex'
    ls_spn_found_tex = \
        getfiles.fp_search_rglob(spt_root=spt_root, ls_srt_subfolders=ls_srt_subfolders,
                                 st_rglob=st_rglob_tex, verbose=verbose)

    for spn_found_tex in ls_spn_found_tex:
        if verbose:
            print(spn_found_tex)
        fl_tex_contents = open(spn_found_tex)
        stf_tex_contents = fl_tex_contents.read()

        # file image folder
        sna_file = Path(spn_found_tex).stem
        fl_tex_contents.close()
        # --listing
        # define new and old
        ls_st_old = ['\\begin{matlabcode}',
                     '\\end{matlabcode}',
                     'matlaboutput',
                     'matlabtitle', 'matlabheading',
                     '[width=\\maxwidth{56.196688409433015em}]{']
        ls_st_new = ['\\begin{lstlisting}',
                     '\\end{lstlisting}',
                     'lstlisting',
                     'subsection', 'subsubsection',
                     '{img/' + sna_file + '_images/']

        # zip and loop and replace
        for old, new in zip(ls_st_old, ls_st_new):
            stf_tex_contents = stf_tex_contents.replace(old, new)
        if verbose:
            print(stf_tex_contents)
        # Open new file
        fl_tex_contents = open(spn_found_tex, 'w')
        fl_tex_contents.write(stf_tex_contents)

        # convert from tex to md
        path_no_suffix = os.path.splitext(spn_found_tex)[0]
        spn_md = path_no_suffix + '.md'
        # print(spn_found_tex)
        # print(spn_found_tex)
        if verbose:
            print(str(['pandoc', '-s', str(spn_found_tex), '-o', str(spn_md)]))

        # atx - headers: so convert all to pounds
        subprocess.Popen(['pandoc',
                          '--atx-headers',
                          '-s', str(spn_found_tex), '-o', str(spn_md)])
        if verbose:
            print(spn_md)


# if __name__ == '__main__':
#     # spt_root_u = "C:/Users/fan/Math4Econ/"
#     # ls_srt_subfolders_u = ['calconevar/', 'derivative/', 'derivative_application/',
#     #                        'matrix_basics/',
#     #                        'opti_firm_constrained/', 'opti_hh_constrained_brsv/',
#     #                        'opti_hh_constrained_brsv_inequality/']
#     # ls_srt_subfolders_u = ['matrix_basics/', 'matrix_application/']
#     # st_rglob_tex_u = '*.tex'
#     # st_rglob_md_u = '*.md'
#     # verbose_u = False
#     #
#     # fp_mlxtex2md(spt_root=spt_root_u, ls_srt_subfolders=ls_srt_subfolders_u,
#     #              st_rglob_tex=st_rglob_tex_u, verbose=verbose_u)
#     #
#     # fp_md2rmd(spt_root=spt_root_u, ls_srt_subfolders=ls_srt_subfolders_u,
#     #           st_rglob_md=st_rglob_md_u, verbose=verbose_u)
#     # from subprocess import Popen, PIPE
#     #
#     # # python -c "from pyfan.util.rmd.mattexmd import fp_mlxtex2md; fp_mlxtex2md(spt_root='C:/Users/fan/Math4Econ/matrix_application/', ls_srt_subfolders=None, st_rglob_tex='twogoods.tex', verbose=True)"
#     #
#     # spg_py_run = "from pyfan.util.rmd.mattexmd import fp_mlxtex2md; fp_mlxtex2md(spt_root='C:/Users/fan/Math4Econ/matrix_application/', ls_srt_subfolders=None, st_rglob_tex='twogoods.tex', verbose=True)"
#     # # spg_py_run = "1+1"
#     #
#     # cmd_popen = Popen(["python", "-c",
#     #                    "\"" + spg_py_run + "\""],
#     #                   stdin=PIPE, stdout=PIPE, stderr=PIPE)
#     # print(cmd_popen)
#     # output, err = cmd_popen.communicate()
#     # print(output.decode('utf-8'))
#     # print(err.decode('utf-8'))
#
#     # fp_mlxtex2md()
#     fp_md2rmd()


'''
Created on Jun 4, 2018

@author: fan
'''

import logging

logger = logging.getLogger(__name__)

import json
from datetime import date, datetime
import numpy as np


def json_serial(obj):
    """JSON serializer for objects not serializable by default json code"""
    if isinstance(obj, (datetime, date)):
        return obj.isoformat()
    if isinstance(obj, (np.int_, np.intc, np.intp,
                        np.int8, np.int16, np.int32,
                        np.int64, np.uint8)):
        return int(obj)
    if isinstance(obj, (np.float_, np.float32)):
        return float(obj)
    if isinstance(obj, np.ndarray):
        return obj.tolist()

    raise TypeError("Type %s not serializable" % type(obj))


def jdump(aws_return_dict, desc='', logger=None, print_here=False):
    strout = json.dumps(aws_return_dict,
                        indent=4,
                        default=json_serial)

    if (print_here):
        print(desc + ':')
        print(strout)

    if (logger is None):
        '''
        default is info logger if there are no loggers specified
        do not specify anything, otherwise plots things I do not want to plot out
        '''
    else:
        logger(desc + '\n:%s',
               json.dumps(aws_return_dict,
                          indent=4,
                          default=json_serial))

    return strout



'''
Created on Aug 10, 2018

@author: fan
'''

import logging
import pyfan.amto.json.json as support_json

logger = logging.getLogger(__name__)


def show_cates(df, varname):
    dtype_name = df[varname].dtype.name
    if (dtype_name == 'category'):
        cates_dict = dict(enumerate(df[varname].cat.categories))
    else:
        cates_dict = 'not categorical,' + varname + ' is ' + dtype_name

    support_json.jdump(cates_dict, df.file_name + ':' + varname,
                       logger=logger.debug, print_here=True)

    return cates_dict



'''
Created on Sep 29, 2013

@author: fan
'''
# import ProjectDisertCredit.Support.Timer import Timer
import time
import datetime as date


def curTimeDiff(startTime=None):
    if (startTime == None):
        return time.time()
    else:
        return (time.time() - startTime)


def getDateTime(timeType=6):
    if (timeType == 1):
        date_time = date.datetime.now().time()

    if (timeType == 2):
        date_time = date.datetime.now()

    if (timeType == 3):
        date_time = date.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

    if (timeType == 4):
        date_time = date.datetime.now().strftime('%Y-%m-%d %H:%M')

    if (timeType == 5):
        date_time = date.datetime.now().strftime('%Y%m_%d_%H%M')

    if (timeType == 6):
        date_time = date.datetime.now().strftime('%Y%m_%d')

    return date_time


class Timer(object):
    def __init__(self, name=None):
        self.name = name

    def __enter__(self):
        self.tstart = time.time()

    def __exit__(self, type, value, traceback):
        if self.name:
            print('[%s]' % self.name)
        print('Elapsed: %s' % (time.time() - self.tstart))


if __name__ == '__main__':
    with Timer('foo_stuff'):
        1 + 1



'''
Created on Oct 5, 2013

@author: fan
'''

import numpy as np
import os as os
# from numpy import genfromtxt

import pyfan.util.timer.timer as timerSup


class csvIO():

    # ===========================================================================
    # saveFileDirectory = 'C:/Users/fan/Documents/Dropbox/Programming/Sandbox/numpyCSVOut.csv'
    # dataOut1 = np.arange(0.0,5.0,1)
    # dataOut2 = np.arange(0,10,1)
    # data1, data2 = np.meshgrid(dataOut1, dataOut2)
    # #===========================================================================
    # # print data1
    # # print data2
    # #===========================================================================
    # dataToSave = np.concatenate((data1,data2))
    # #===========================================================================
    # # print 'printing data to be saved'
    # # print dataAll
    # #===========================================================================
    # np.savetxt(saveFileDirectory, dataToSave, delimiter=',')   # X is an array
    # 
    # dataOut = np.loadtxt(saveFileDirectory, delimiter=',')
    # #===========================================================================
    # # print 'printing from loadtxt'
    # # print dataOut
    # #===========================================================================
    # ===========================================================================

    def numerical_parmlist_stringify(self, trueParameters):
        out_str = ''
        for key in trueParameters.keys():
            val = trueParameters[key]
            if (val > 1):
                out_str += str(int(trueParameters[key]))
            else:
                out_str += str(int(trueParameters[key] * 100))
        return out_str

    def createDirectory(self, save_file_directory, addName=''):
        addName = addName.replace('0.', 'p')
        addName = addName.replace('.', 'p')
        addName = addName.replace('\'', '')
        addName = addName.replace(':', '')
        addName = addName.replace(' ', '')
        addName = addName.replace(',', '')
        addName = addName.replace('{', '')
        addName = addName.replace('}', '')
        addName = addName.replace('-', 'n')
        addName = addName.replace(']', '')
        addName = addName.replace('[', '')
        addName = addName.replace('array(p', '')
        addName = addName.replace(')', '')

        fullname = save_file_directory + addName[0:60] + '/'

        if not os.path.exists(fullname):
            try:
                os.makedirs(fullname)
            except FileExistsError:
                print("Tried to concurrently create folders")

        return fullname

    def saveCSV(self, saveFileDirectory, dataToSave, header=''):
        np.savetxt(saveFileDirectory, dataToSave, delimiter=',', header=header)  # X is an array

    def loadCSV(self, saveFileDirectory, unpack=True):
        return np.loadtxt(saveFileDirectory, delimiter=',', unpack=True)
        # return genfromtxt(saveFileDirectory, delimiter=",",unpack=True)

    def loadText(self, saveFileDirectory):
        f = open(saveFileDirectory, 'r')
        fileOut = f.read()
        f.close()
        return fileOut

    def print_data_summary(self, numpy_matrix, numpy_matrix_name, saveFileDirectory=None, \
                           printSummary=True, printFullMatrix=False, saveText=True, printConsole=True,
                           printFullArray=False, precision=None, linewidth=200):

        np.set_printoptions(linewidth=linewidth)

        if (precision != None):
            np.set_printoptions(precision=precision)
        else:
            np.set_printoptions(precision=2)

        np.set_printoptions(suppress=True)
        if (printFullArray == True):
            np.set_printoptions(threshold=np.nan)

        #         print ''
        # =======================================================================
        # try:
        #     print name+'.shape'
        #     print name.shape
        # except:
        #     pass
        # try:
        #     print 'len('+name+'), len('+name+'[0])'
        #     print len(name), len(name[0])
        # except:
        #     pass        
        # =======================================================================

        #         try:

        if (printSummary == True):
            try:
                summary_line = '\n\n' + numpy_matrix_name + '\n' + \
                               numpy_matrix_name + '.shape:' + str(
                    numpy_matrix.shape) + ', np.mean(' + numpy_matrix_name + '):' + str(
                    np.mean(numpy_matrix)) + ', np.sd(' + numpy_matrix_name + '):' + str(np.std(numpy_matrix)) + \
                               '\nmin(' + numpy_matrix_name + '):' + str(
                    min(numpy_matrix)) + ', max(' + numpy_matrix_name + '):' + str(max(numpy_matrix)) + '\n'
                if (saveText == True):
                    self.saveText(saveFileDirectory, str(summary_line), type='a', addTimer=False, lineBreak=False,
                                  doubleBreak=False, printConsole=False)

                if (printConsole == True):
                    print(summary_line)
            except:
                pass
        else:
            pass

        try:
            if (printFullMatrix == True):

                if (saveText == True):
                    np.set_printoptions(precision=precision)
                    self.saveText(saveFileDirectory, '\n\n' + numpy_matrix_name + '\n' + str(numpy_matrix) + '\n',
                                  type='a', addTimer=False, lineBreak=False, doubleBreak=False, printConsole=False)

                if (printConsole == True):
                    print(numpy_matrix_name)
                    np.set_printoptions(precision=precision)
                    print(numpy_matrix)
        except:
            pass

    #         except:
    #             print 'failed to print ', numpy_matrix_name

    def saveText(self, saveFileDirectory, dataToSave, type='w',
                 addTimer=False, lineBreak=True, doubleBreak=False, printConsole=True, precision=None, linewidth=200):

        # ===========================================================================
        # The first argument is a string containing the filename.
        # The second argument is another string containing a few
        # characters describing the way in which the file will be used.
        # mode can be
        # 'r' when the file will only be read,
        # 'w' for only writing (an existing file with the same name will be erased), and
        # 'a' opens the file for appending; any data written to the file is automatically added to the end.
        # 'r+' opens the file for both reading and writing.
        #
        # On Windows, 'b' appended to the mode opens the file in binary mode,
        # so there are also modes like 'rb', 'wb', and 'r+b'.
        # Python on Windows makes a distinction between text and binary files;
        # the end-of-line characters in text files are automatically altered slightly when data is read or written.
        # This behind-the-scenes modification to file data is fine for ASCII text files,
        # Be very careful to use binary mode when reading and writing such files.
        # On Unix, it doesnt hurt to append a 'b' to the mode,
        # so you can use it platform-independently for all binary files.
        # ===========================================================================

        np.set_printoptions(linewidth=linewidth)

        if (precision != None):
            np.set_printoptions(precision=precision)
        else:
            np.set_printoptions(precision=2)

        if (printConsole == True):
            print(dataToSave)

        f = open(saveFileDirectory, type)

        if (addTimer == True):
            current_time = timerSup.getDateTime(timeType=3)
            f.write(current_time + '\n')

        if (doubleBreak == True):
            f.write('\n\n' + str(dataToSave) + '\n')
        elif (lineBreak == True):
            f.write(str(dataToSave) + '\n')
        else:
            f.write(str(dataToSave))

        f.close()
